/* @(#) $Id: ALRXmlDSigSvcs.js,v 1.1 2013/05/23 22:08:26 subrata Exp $ */

/* ***** BEGIN LICENSE BLOCK *****
 * The Initial Developer of the Original Code is
 * Subrata Mazumdar, Avaya Labs Research, Avaya Inc.
 * Portions created by the Initial Developer are Copyright (C) 2010
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Subrata Mazumdar (mazum@avaya.com)
 *
 * ***** END LICENSE BLOCK ***** */

/*
 *
 * XPCOM Implementation references:
 * 	https://developer.mozilla.org/en/XPCOMUtils.jsm  (For FF3+)
 * 	https://developer.mozilla.org/en/XPCOM/XPCOM_changes_in_Gecko_2.0 (FF4.0)
 *
 */

/*
 *  ATTENTION
 *
 *  Firefox is very very unforgiving to syntax errors in XPCOM Javascript
 *  files. If this file does not parse properly once, it might not be
 *  loaded again even if you restart Firefox. Watch the JavaScript
 *  Console for any error message (only logged the first time the file
 *  fails to parse).
 *
 *  You might have to uninstall the whole extension, restart Firefox, 
 *  reinstall the extension (with fixes), and then restart Firefox.
 */
 

let Cu = Components.utils;
let Ci = Components.interfaces;
let Cc = Components.classes;
let Cr = Components.results;

/* Load XPCOMUtils Javascript module */
Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");


function ALRXmlDSigSvcs_log(aMessage) {
    /*
    var _msg = "ALRXmlDSigSvcs: " + aMessage + "\n";
    dump(_msg);
    */
}



/*----------------------------------------------------------------------
 * The ALRXmlDSigSvcs Component, implemented as a Javascript function.
 *----------------------------------------------------------------------
 */

//class constructor
function ALRXmlDSigSvcs() {
    ALRXmlDSigSvcs_log("ALRXmlDSigSvcs():..................Start.");
    /*
     *  This is a XPCOM-in-Javascript trick: Clients using an XPCOM
     *  implemented in Javascript can access its wrappedJSObject field
     *  and then from there, access its Javascript methods that are
     *  not declared in any of the IDL interfaces that it implements.
     *
     *  Being able to call directly the methods of a Javascript-based
     *  XPCOM allows clients to pass to it and receive from it
     *  objects of types not supported by IDL.
     */
    this.wrappedJSObject = this;
    
    this._initialized = false;

    ALRXmlDSigSvcs_log("ALRXmlDSigSvcs():..................End.");
};

//class definition
ALRXmlDSigSvcs.prototype = {

    /** 
     * .classID is required for generateNSGetFactory to work correctly.
     * Make sure this CID matches the "component" in your .manifest file.
     */
    classID : Components.ID("{713b9036-bf51-4508-9ac4-7c9fb245c6ec}"),

    /**
     * .classDescription and .contractID are only used for
     * * backwards compatibility with Gecko 1.9.2 and
     * * XPCOMUtils.generateNSGetModule.
     */
    classDescription: "XML DSIG Services Access JS Impl", // any human-readable string
    contractID: "@avaya.com/avxmldsig/js/avxmldsigsvcs;1",

    /**
     * List all the interfaces your component supports.
     * @note nsISupports is generated automatically; you don't need to list it.
     */
    QueryInterface : XPCOMUtils.generateQI([
    			Ci.nsIDOMGlobalPropertyInitializer,
			Ci.nsISupports
			]),

    mVersion: "0.0.77",
    get version() { return this.mVersion; },

    init : function(aWindow) 
    {
     	ALRXmlDSigSvcs_log("ALRXmlDSigSvcs.init():................Start.");

	this.window = XPCNativeWrapper.unwrap(aWindow);
	this.sandbox = Cu.Sandbox(this.window,
				{ sandboxPrototype: this.window, wantXrays: false });

	this.avXmlDSigSvcs = {};
	try {
            var x509CertDBUtil = Components.classes["@avaya.com/pkm/x509certdbutil;1"]
				.getService(Components.interfaces.alrIX509CertDBUtil);
            var xmlDSigTool = Components.classes["@avaya.com/pkm/xmldsigtool;1"]
				.getService(Components.interfaces.alrIXPXmlDSigTool);
 	    var sigEncXmlEncSvc = Components.classes["@avaya.com/pkm/xmlencsvc;1"]
				.getService(Components.interfaces.alrIXPXmlEncSvc);
	    var samlTokenSvc = Components.classes["@avaya.com/pkm/samltokensvc;1"]
				.getService(Components.interfaces.alrIXPSamlTokenSvc);
	    var uuidGenerator = Components.classes["@avaya.com/pkm/uuidgenerator;1"]
				.getService(Components.interfaces.alrIUUIDGenerator);

	    this.avXmlDSigSvcs = {
		"x509CertDBUtil" : x509CertDBUtil,
		"sigEncXmlEncSvc" : sigEncXmlEncSvc,
		"samlTokenSvc" : samlTokenSvc,
		"uuidGenerator" : uuidGenerator,
		"xmlDSigTool" : xmlDSigTool,
		"__exposedProps__" : {
		    "x509CertDBUtil" : "r",
		    "sigEncXmlEncSvc" : "r",
		    "samlTokenSvc" : "r",
		    "uuidGenerator" : "r",
		    "xmlDSigTool" : "r"
		    }
		};
	} catch (ex) {
     	    ALRXmlDSigSvcs_log("ALRXmlDSigSvcs.init(): failed - ex: " + ex);
	}

     	ALRXmlDSigSvcs_log("ALRXmlDSigSvcs.init():................End.");
	return this.avXmlDSigSvcs;
    },

    /*
    // TODO: test code: 
    */
    test1 : function()
    {
        alert(Components.classes["@avaya.com/avxmldsig/js/avxmldsigsvcs;1"].getService(Components.interfaces.nsISupports).wrappedJSObject.version);
        alert(window.avxmldsigsvcs.xmlDSigTool);
    },

    lastMethod : function()
    {
    }
};

/**
 * XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
 * XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
 */

var NSGetFactory = XPCOMUtils.generateNSGetFactory([ALRXmlDSigSvcs]);

