/* @(#) $Id: HtmlDigitalSigWatcher.js,v 1.10 2012/12/05 19:02:02 subrata Exp $ */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is  Avaya Labs Research, Avaya Inc. Code.
 *
 * The Initial Developer of the Original Code is
 *     Subrata Mazumdar, Avaya Labs Research, Avaya Inc.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Subrata Mazumdar (mazum@avaya.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


// class constructor
function HtmlDigitalSigWatcherX(aWatcherId) {
    this._watcherId = aWatcherId;
};

var htmlDigitalSigWatcherCount = 0;

//class definition
HtmlDigitalSigWatcherX.prototype = {
}

// Install a timeout handler to install the interval routine

var HtmlDigitalSigWatcher = {
	_isSigned: false, 
	_sigProfile: null, 
	_sigDocURI: null, 

	_signerX509Cert: null, 
	_isVerified: false, 
	_isValidated: false, 
	_proxyEnabled: false, 
	_htmlDSigValidatorElem: null,

	_watcherId: null,
	_maxLogLevel: 2,

	log: function(level, msg)
	{
	    if (level > this._maxLogLevel) {
	    	return;
	    }
	    dump(msg + "\n");
	},

	logError: function(msg)
	{
	    this.log(2, msg);
	},
	logTrace: function(msg)
	{
	    this.log(4, msg);
	},
	logDebug: function(msg)
	{
	    this.log(8, msg);
	},

	startup: function(ev)
	{
	    this.logTrace("HtmlDigitalSigWatcher.startup():.............Start.");

	    this.proxyEnabled();

	    this.handlePageLoad(ev);

	    this.logTrace("HtmlDigitalSigWatcher.startup():.............End.");
	},

	proxyEnabled: function()
	{
	    this._proxyEnabled = false;

	    var Prefs = Components.classes["@mozilla.org/preferences-service;1"]
                   		.getService(Components.interfaces.nsIPrefService);
	    var prefsBranch = Prefs.getBranch("");

	    var proxyType = 0;
	    try {
	    	proxyType = prefsBranch.getIntPref("network.proxy.type");
	    } catch (e) {
	    	// nothing
      	    }
	    this.logDebug("HtmlDigitalSigWatcher.htmlDsigValidationEnabled(): proxyType: " + proxyType + "\n");

	    // prefsBranch = Prefs.getBranch("");
	    var proxyOverride = false;
	    try {
	    	proxyOverride = prefsBranch.getBoolPref("extensions.avxmldsig.htmldsig.proxy.override");
	    } catch (e) {
	    	// nothing
      	    }
	    this.logDebug("HtmlDigitalSigWatcher.htmlDsigValidationEnabled(): proxyOverride: " + proxyOverride + "\n");

	    if (!proxyOverride && (proxyType > 0)) {
	    	this._proxyEnabled = true;
	    }
	    return this._proxyEnabled;
	},

	htmlDsigValidationEnabled: function()
	{
	    this.logTrace("HtmlDigitalSigWatcher.htmlDsigValidationEnabled():.............Start.");

	    var htmldsigStatusBarPanel = document.getElementById("htmldsig-status-bar");

	    var Prefs = Components.classes["@mozilla.org/preferences-service;1"]
                   		.getService(Components.interfaces.nsIPrefService);
	    var prefsBranch = Prefs.getBranch("extensions.avxmldsig.htmldsig.");

	    var htmldsigEnabled = false;
	    try {
	    	htmldsigEnabled = prefsBranch.getBoolPref("enable");
	    } catch (e) {
	    	// nothing
      	    }
	    if (!htmldsigEnabled) {
	    	htmldsigStatusBarPanel.hidden = true;
	    	this.logTrace("HtmlDigitalSigWatcher.htmlDsigValidationEnabled():.............End(1).");
	    	return false;
	    }
	    prefsBranch = Prefs.getBranch("extensions.avxmldsig.htmldsig.validation.");
	    var htmldsigValidationEnabled = false;
	    try {
	    	htmldsigValidationEnabled = prefsBranch.getBoolPref("enable");
	    } catch (e) {
	    	// nothing
      	    }
	    if (!htmldsigValidationEnabled) {
	    	htmldsigStatusBarPanel.hidden = true;
	    	this.logTrace("HtmlDigitalSigWatcher.htmlDsigValidationEnabled():.............End(2).");
	    	return false;
	    }

	    this.logTrace("HtmlDigitalSigWatcher.htmlDsigValidationEnabled():.............End.");
	    return true;
	},

	hanldeTabSelect: function(ev)
	{
	    // this.logTrace("HtmlDigitalSigWatcher.hanldeTabSelect():.............Start.");

	    // browser is the XUL element of the browser that's just been selected
	    var browser = gBrowser.getBrowserAtIndex(gBrowser.mTabContainer.selectedIndex);

	    // Getting document of currently selected tab
	    var htmlDoc = gBrowser.selectedBrowser.contentDocument;

	    this.refreshInformation();

	    // this.logTrace("HtmlDigitalSigWatcher.hanldeTabSelect():.............End.");
	},

	handlePageLoad: function(ev)
	{
	    // this.logTrace("HtmlDigitalSigWatcher.handlePageLoad():.............Start.");

	    var htmlDoc = null;
            if (ev.originalTarget instanceof HTMLDocument) {
                htmlDoc = ev.originalTarget;
                if (ev.originalTarget.defaultView.frameElement) {
		    // Frame within a tab was loaded. doc should be the root document of
		    // the frameset. If you don't want do anything when frames/iframes
		    // are loaded in this web page, uncomment the following line:
		    // return;

		    // Find the root document:
		    while (htmlDoc.defaultView.frameElement) {
			htmlDoc=htmlDoc.defaultView.frameElement.ownerDocument;
		    }
		}
	    }

	    this.refreshInformation();

	    // this.logTrace("HtmlDigitalSigWatcher.handlePageLoad():.............End.");
	},

	refreshInformation: function()
	{
	    // this.logTrace("HtmlDigitalSigWatcher.refreshInformation():.............Start.");

	    this._isSigned =  false;
	    this._sigProfile = null;
	    this._sigDocURI =  null;
	    this._isVerified =  false;
	    this._isValidated =  false;
	    this._signerX509Cert =  null;

	    var htmlDocURI = gBrowser.selectedBrowser.currentURI;
	    var htmlDocURISpec = htmlDocURI.spec;
	    // this.logDebug("HtmlDigitalSigWatcher.refreshInformation(): htmlDocURI.spec: " + htmlDocURI.spec + "");
	    if (!  ((htmlDocURISpec.indexOf("http://") == 0)
	    	    || (htmlDocURISpec.indexOf("https://") == 0)
		    || (htmlDocURISpec.indexOf("file://") == 0))
		) {
	    	return;
	    }
	    // this.logTrace("HtmlDigitalSigWatcher.refreshInformation():.............10.");
	    
	    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
					.getService(Components.interfaces.nsIIOService);
	    var htmlDocBaseURI = null;
	    try {
	    	htmlDocBaseURI = ioService.newURI(".", null, htmlDocURI);
	    } catch (ex) {
	    }
	    if (!htmlDocBaseURI) {
	    	return;
	    }

	    // this.logTrace("HtmlDigitalSigWatcher.refreshInformation():.............30.");

	    var htmldsigStatusBarPanel = document.getElementById("htmldsig-status-bar");
	    if (!htmldsigStatusBarPanel) {
		return;
	    }

	    if (!this._htmlDSigValidatorElem) {
		const xulNS = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"
	    	this._htmlDSigValidatorElem = document.getElementById("htmldsig-validator");
		/*
	    	if (!this._htmlDSigValidatorElem) {
	    	    this._htmlDSigValidatorElem = htmldsigStatusBarPanel.ownerDocument.createElementNS(
					xulNS,
					"HTMLSignatureEditor"
					);
		}
		*/
	    }


	    htmldsigStatusBarPanel.hidden = true;
	    htmldsigStatusBarPanel.removeAttribute("label");
	    htmldsigStatusBarPanel.removeAttribute("tooltiptext");
	    htmldsigStatusBarPanel.removeAttribute("sigProfile");
	    htmldsigStatusBarPanel.removeAttribute("signed");
	    htmldsigStatusBarPanel.removeAttribute("verified");
	    htmldsigStatusBarPanel.removeAttribute("validated");
	    htmldsigStatusBarPanel.removeAttribute("signedBy");
	    htmldsigStatusBarPanel.signerCert = null;

	    // this.logTrace("HtmlDigitalSigWatcher.refreshInformation():.............40.");

	    var htmlDoc = gBrowser.selectedBrowser.contentDocument;

	    const xhtmlNsURI = "http://www.w3.org/1999/xhtml";
	    {
		var htmlDocSigProfile = null;

		// Look for the HTML Signature resource URI using the 'link' element 
		var htmlLinkElemList = htmlDoc.getElementsByTagNameNS(xhtmlNsURI, "link");
		if (htmlLinkElemList.length <= 0) {
		    htmlLinkElemList = htmlDoc.getElementsByTagName("link");
		}
	    	// this.logDebug("HtmlDigitalSigWatcher.refreshInformation(): htmlLinkElemList.length: " + htmlLinkElemList.length + "");

		for (var i = 0; i < htmlLinkElemList.length; i++) {
		    var htmlLinkElem = htmlLinkElemList.item(i);

		    var htmlLinkRelAttr = htmlLinkElem.getAttribute("rel");
		    if ((htmlLinkRelAttr != "dsig-full") && (htmlLinkRelAttr != "dsig-simple")) {
			continue;
		    }
		    var htmlLinkHrefAttr = htmlLinkElem.getAttribute("href");
		    if (!htmlLinkHrefAttr) {
			continue;
		    }

		    this._isSigned =  true;
		    this._sigProfile = htmlLinkRelAttr;
		    
		    var sigDocURI = htmlLinkHrefAttr;
		    if ((sigDocURI.indexOf("http://") == 0) 
		        || (sigDocURI.indexOf("https://") == 0) 
		        || (sigDocURI.indexOf("file://") == 0) 
			) {
			this._sigDocURI =  sigDocURI;
		    }
		    else {
			if (htmlDocBaseURI) {
			    sigDocURI = htmlDocBaseURI.resolve(sigDocURI);
			}
			this._sigDocURI =  sigDocURI;
		    }
		    break;
		}
	    }
	    this.logDebug("HtmlDigitalSigWatcher.refreshInformation(): " + 
				"_proxyEnabled: " + this._proxyEnabled + ", " + 
				"_isSigned: " + this._isSigned + ", " + 
				"_sigProfile: " + this._sigProfile + ", " + 
				"htmlDocBaseURI: " + htmlDocBaseURI.spec + ", " + 
				"_sigDocURI: " + this._sigDocURI + " " + 
	    			"");

	    if (!this._isSigned) {
		return;
	    }
	    htmldsigStatusBarPanel.hidden = false;

	    this.proxyEnabled();

	    htmldsigStatusBarPanel.label = "Signed";
	    htmldsigStatusBarPanel.tooltipText = 
	    		"Signed by " +
			(this._signerX509Cert ? this._signerX509Cert.commonName : "Unknown") + ", " +   
			"profile: " + this._sigProfile + ", " + 
			"verified? " + (this._isVerified ? "yes" : "not performed yet") + ", " + 
			"validated? " + (this._isValidated ? "yes" : "not performed yet") + ", " + 
			"";

	    // TODO: Use a background thread to update the status bar.
	    window.setTimeout(
	    	function() { HtmlDigitalSigWatcher.updateStatusBar(); },
		1000
		);

	    /*
	    // Source: http://developer.mozilla.org/en/docs/The_Thread_Manager
	    var background = Components.classes["@mozilla.org/thread-manager;1"].getService().newThread(0);
	    var main = Components.classes["@mozilla.org/thread-manager;1"].getService().mainThread;
	    background.dispatch(new workingThread(1, 5000000), background.DISPATCH_NORMAL);
	    */
	},

	updateStatusBar: function()
	{
	    var htmldsigStatusBarPanel = document.getElementById("htmldsig-status-bar");

	    this.logTrace("HtmlDigitalSigWatcher.updateStatusBar():.............Start.");
	    var isValidated =  false;
	    
	    var isVerified = false;
	    isVerified = this.verifyHtmlDocSignature(this._sigDocURI);

	    var htmlDocURI = gBrowser.selectedBrowser.currentURI;
	    var htmlDocURISpec = htmlDocURI.spec;
    	    var validHtmlDocSig = false;
	    if (this._htmlDSigValidatorElem) {
	    	validHtmlDocSig = this._htmlDSigValidatorElem.validateSignedHtmlURI(htmlDocURISpec);
	    }

	    this._isValidated = false;
	    if (isVerified && validHtmlDocSig) {
	    	this._isValidated = true;
	    }
	    this.logDebug("HtmlDigitalSigWatcher.updateStatusBar(): isVerified: " + isVerified + " validHtmlDocSig: " + validHtmlDocSig);

	    // Now initialize the attributes and status of the htmldsig-status-bar panel.
    
	    htmldsigStatusBarPanel.setAttribute("sigProfile", this._sigProfile);
	    if (this._isSigned) {
	    	htmldsigStatusBarPanel.setAttribute("signed", this._isSigned);
	    }
	    htmldsigStatusBarPanel.setAttribute("signedBy", (this._signerX509Cert ? this._signerX509Cert.commonName : "Unknown"));
	    htmldsigStatusBarPanel.signerCert = this._signerX509Cert;


	    if (this._isValidated) {
	    	htmldsigStatusBarPanel.setAttribute("validated", this._isValidated);
	    }
	    if (this._isVerified) {
	    	htmldsigStatusBarPanel.setAttribute("verified", this._isVerified);
	    }

	    htmldsigStatusBarPanel.label = "Signed";
	    htmldsigStatusBarPanel.tooltipText = 
	    		"Signed by " +
			(this._signerX509Cert ? this._signerX509Cert.commonName : "Unknown") + ", " +   
			"profile: " + this._sigProfile + ", " + 
			"verified? " + (this._isVerified ? "yes" : (this._proxyEnabled?"not performed":"failed")) + ", " + 
			"validated? " + (this._isValidated ? "yes" : (this._proxyEnabled?"not performed":"failed")) + ", " + 
			"";

	    this.logTrace("HtmlDigitalSigWatcher.updateStatusBar():.............End.");
	},

	verifyHtmlDocSignature: function(aSigXmlDocURISpec)
	{
	    this.logTrace( "HtmlDigitalSigWatcher.verifyHtmlDocSignature():...............Start.\n");

	    var ioService = Components.classes["@mozilla.org/network/io-service;1"]
					.getService(Components.interfaces.nsIIOService);
	    var x509CertDBUtil = Components.classes["@avaya.com/pkm/x509certdbutil;1"]
					.getService(Components.interfaces.alrIX509CertDBUtil);
	    var xmlDSigTool = Components.classes["@avaya.com/pkm/xmldsigtool;1"]
					.getService(Components.interfaces.alrIXPXmlDSigTool);
	    if (!xmlDSigTool) {
		return false;
	    }
	    this.logTrace( "HtmlDigitalSigWatcher.verifyHtmlDocSignature():...............10.\n");

	    var signerCert = null;
	    var sigXmlDocURI = ioService.newURI(aSigXmlDocURISpec, null, null);
	    var sigXmlDoc = null;
	    try {
	    	sigXmlDoc = xmlDSigTool.readXMLDocFromURI(sigXmlDocURI);
	    } catch (ex) {
	    	this.logError( "HtmlDigitalSigWatcher.verifyHtmlDocSignature(): xmlDSigTool.readXMLDocFromURI() failed for " + sigXmlDocURI.spec + " - ex: " + ex);
		return false;
	    }

	    try {
		signerCert = x509CertDBUtil.findX509CertBySignatureDoc(sigXmlDoc);
	    } catch (ex) {
	    	this.logError( "HtmlDigitalSigWatcher.verifyHtmlDocSignature(): x509CertDBUtil.findX509CertBySignatureDoc() failed - ex: " + ex);
		return false;
	    }
	    this._signerX509Cert = signerCert;

	    this.logTrace( "HtmlDigitalSigWatcher.verifyHtmlDocSignature():...............20.\n");

	    var /* nsIPersistentProperties */ sigParamProps = null;
	    sigParamProps = Components.classes["@mozilla.org/persistent-properties;1"].
			createInstance(Components.interfaces.nsIPersistentProperties);
	    var signerCertObj = new Object();
	    var isPassed = false;
	    try {
		signerCertObj.value = null;
		if (signerCert) {
		    signerCertObj.value = signerCert;
		}
		// Verify the signature only if the browser proxy is not set
		// XERCES-C++ cannot load document through proxies. As a result 
		// Apaches XML_SEC fails to load the XML document for the detached signature.
		// TODO:  Figure out how to enable proxies for XERCES-C++
	    	if (!this._proxyEnabled) {
	    	    this.logTrace( "HtmlDigitalSigWatcher.verifyHtmlDocSignature():...............50.\n");
		    isPassed = xmlDSigTool.verifySignedXMLDocByX509Cert(
				    sigXmlDoc,
				    signerCertObj,
				    sigParamProps
				    );
	    	}
	    } catch (ex) {
		this.logDebug("Failed to verify signed archive file - ex : " + ex + "");
		isPassed = false;
	    }

	    this._isVerified = isPassed;
	    this._signerX509Cert = signerCertObj.value;
	    
	    this.logTrace( "HtmlDigitalSigWatcher.verifyHtmlDocSignature():...............End.\n");
	    return isPassed;
	},

	showVerifySignatureDialog: function(ev)
	{
	    this.logTrace("HtmlDigitalSigWatcher.showVerifySignatureDialog():.............Start.");

	    // this.hanldeTabSelect(ev);

	    if (!this._sigDocURI) {
	        return;
	    }

            var argc = 0;
            var params = Components.classes["@mozilla.org/embedcomp/dialogparam;1"]
                                    .createInstance(Components.interfaces.nsIDialogParamBlock);
            params.SetString(argc, this._sigDocURI); argc++;
            params.SetInt(0, argc);
            window.openDialog('chrome://xmldsigffext/content/xmldsig/XmlDigitalSigViewer.xul',
                                    '_blank', 'chrome,centerscreen,resizable,modal,titlebar', params);

            var retVal = params.GetInt(0);
            if (retVal == 0) {
                return;
            }

	    this.logTrace("HtmlDigitalSigWatcher.showVerifySignatureDialog():.............End.");
	}
}


/*
Source:  Detecting page load: 
http://developer.mozilla.org/en/docs/Code_snippets:Tabbed_browser#Detecting_page_load
http://developer.mozilla.org/en/docs/Code_snippets:On_page_load
*/

/*
// Install load handler if the XML Digital Signature  Service is present.
if (Components.interfaces.alrIXPXmlDSigTool) {
}
*/

window.addEventListener(
  "load",
  function (e) {
    // Add a callback to be run every time a document loads.
    // note that this includes frames/iframes within the document
    if (!HtmlDigitalSigWatcher.htmlDsigValidationEnabled()) {
	var htmldsigStatusBarPanel = document.getElementById("htmldsig-status-bar");
	htmldsigStatusBarPanel.hidden = true;
        return;
    }
    HtmlDigitalSigWatcher.startup(e);
    gBrowser.addEventListener("load", function(e) { HtmlDigitalSigWatcher.handlePageLoad(e); }, true);

    // Detecting tab selection
    var container = gBrowser.tabContainer;
    container.addEventListener("TabSelect", function(e) { HtmlDigitalSigWatcher.hanldeTabSelect(e); }, false);
    container.addEventListener("TabClose", function(e) { HtmlDigitalSigWatcher.hanldeTabSelect(e); }, false);
  },
  false
);

