/* @(#) $Id: XmlDigitalSigTool.js,v 1.22 2013/05/23 22:11:30 subrata Exp $ */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is  Avaya Labs Research, Avaya Inc. Code.
 *
 * The Initial Developer of the Original Code is
 *     Subrata Mazumdar, Avaya Labs Research, Avaya Inc.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Subrata Mazumdar (mazum@avaya.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var gParams = null;
var gXmlDSigJsonParams = {};

var gTabboxElem;
var gSignTabElem;
var gVerifyTabElem;

var gPreviewCmdButtonElem;
var gSignCmdButtonElem;
var gVerifyCmdButtonElem;

var gXmlParser;
var gXmlSerializer;

var /* alrIFileUtil	  */ gFileUtil = null;
var /* nsIX509CertDB	  */ gCertDB = null;
var /* alrIXPXmlDSigTool  */ gXmlDSigTool = null;
var /* alrIX509CertDBUtil */ gX509CertDBUtil = null;

var alreadyInit = false;

const XPILoadErrorMessage1 = "Failed to load shared librararies and/or register XPCOM componenets for XML Digital Signature tool ."
const XPILoadErrorMessage2 = "Make sure that DLLs (shared libraries) from Apache Xerces-C++ and Xalan-C++ (for FF2 only) are in your PATH (LD_LIBRARY_PATH) environment variable.";


/* nsIFile */
function xmldsigffext_getUserProfileDir()
{
    // First get the directory service and query interface it to
    // nsIProperties
    var dirService = Components.classes['@mozilla.org/file/directory_service;1']
  			.getService(Components.interfaces.nsIProperties);
    var profileDir = dirService.get("ProfD", Components.interfaces.nsIFile);
    return profileDir;
}

/* nsIFile */
function xmldsigffext_getToolAddonDir()
{
    var profileDir = xmldsigffext_getUserProfileDir();
    var toolAddonDir = profileDir.clone();
    toolAddonDir.append("extensions");
    toolAddonDir.append("{bcb4f322-a177-4ecd-9c80-835d96e1e595}");
    return toolAddonDir;
}


function xmldsigffext_getTagertPlatformName()
{
    // XmlDigitalSigTool.logDebug("xmldsigffext_getTagertPlatformName(): userAgent: " +  navigator.userAgent + "");
    
    var isWin = (navigator.userAgent.indexOf("Win") != -1);
    var isMac = (navigator.userAgent.indexOf("Mac") != -1);
    var isLinux = (navigator.userAgent.indexOf("Linux") != -1);
    var isLinux64 = false;
    if (isLinux) {
    	isLinux64 = (navigator.userAgent.indexOf("x86_64") != -1);
    }

    var targetPlatformName = "";
    if (isWin) {
    	targetPlatformName = "WINNT_x86-msvc";
    }
    else if (isLinux) {
    	if (isLinux64) {
    	    targetPlatformName = "Linux_x86_64-gcc3";
	}
	else {
    	    targetPlatformName = "Linux_x86-gcc3";
	}
    }
    else if (isMac) {
    	targetPlatformName = "Darwin_x86-gcc3";
    }
    return targetPlatformName;
}

/* nsIFile */
function xmldsigffext_getToolPlatformDepCompDir()
{
    var toolDepcompDir = xmldsigffext_getToolAddonDir();
    var targetPlatformName = xmldsigffext_getTagertPlatformName();
    if (targetPlatformName == "") {
    	return toolDepcompDir;
    }
    toolDepcompDir.append("platform");
    toolDepcompDir.append(targetPlatformName);
    toolDepcompDir.append("depcomps");
    return toolDepcompDir;
}

function xmldsigffext_checkForXPCOMServices()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_checkForXPCOMServices():................Start.");

    var result = false;
	var errmsg1 = XPILoadErrorMessage1;
	var errmsg2 = XPILoadErrorMessage2;
	var platformDepCompDir = xmldsigffext_getToolPlatformDepCompDir();
	var platformDepCompDirPath = platformDepCompDir.path;
	var errmsg3 = "";
    	var isMac = (navigator.userAgent.indexOf("Mac") != -1);
	if (isMac) {
	    errmsg3 = "Adding the following path to your DYLD_LIBRARY_PATH : " + platformDepCompDirPath  + " before installing the add-on may help.";
	}

    if (!Components.classes["@avaya.com/pkm/x509certdbutil;1"]
	|| !Components.classes["@avaya.com/pkm/xmldsigtool;1"]
	) {
    	XmlDigitalSigTool.logError(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "");
    	alert(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "");
    	throw errmsg1;
    }
    	
    if (!Components.interfaces.nsIX509CertDB
		|| !Components.interfaces.alrIX509CertDBUtil
		|| !Components.interfaces.alrIXPXmlDSigTool 
		) {
    	XmlDigitalSigTool.logError(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "");
    	alert(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "");
    	throw errmsg1;
    }

    XmlDigitalSigTool.logTrace("xmldsigffext_checkForXPCOMServices():................End.");
    return true;
}

function xmldsigffext_initOnLoad()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_initOnLoad():................Start.");

    if (alreadyInit) {
    	return;
    }
    alreadyInit = true;

    xmldsigffext_isSupportedOS();

    xmldsigffext_checkForXPCOMServices();

    try {

    	gCertDB = Components.classes["@mozilla.org/security/x509certdb;1"].
			getService(Components.interfaces.nsIX509CertDB);
	gX509CertDBUtil = Components.classes["@avaya.com/pkm/x509certdbutil;1"].
    			getService(Components.interfaces.alrIX509CertDBUtil);
    	gXmlDSigTool = Components.classes["@avaya.com/pkm/xmldsigtool;1"].
    			getService(Components.interfaces.alrIXPXmlDSigTool);
    	gFileUtil = Components.classes["@avaya.com/avxmldsig/js/fileutil;1"].
			getService(Components.interfaces.alrIFileUtil);
    } catch (ex) {
    	alert("xmldsigffext_initOnLoad(): failed to create instance of alrIXPXmlDSigTool - ex: " + ex + "");
    	XmlDigitalSigTool.logError("xmldsigffext_initOnLoad(): failed to create instance of alrIXPXmlDSigTool - ex: " + ex + "");
	XmlDigitalSigTool.logError(XPILoadErrorMessage2 + "");
	throw ex;
    }
    if (gXmlDSigTool == null) {
    	alert("xmldsigffext_initOnLoad(): failed to create instance of alrIXPXmlDSigTool.\n" + XPILoadErrorMessage2);
    	XmlDigitalSigTool.logError("xmldsigffext_initOnLoad(): failed to create instance of alrIXPXmlDSigTool.\n" + XPILoadErrorMessage2);
    	throw XPILoadErrorMessage2;
    }

    /*
    {
	try  {
	    XmlDigitalSigTool.logDebug("gFileUtil: " + gFileUtil + "");
	    if (gFileUtil) {
		var inXmlFilePath = "/Users/subrata/XMLDOC/ATestDoc.xml";
		var inXmlFileData = this.readDataFromFilePath(inXmlFilePath);
		gXmlDSigTool.test1(inXmlFileData, inXmlFileData);
	   	// gFileUtil.test1();
	    }
	} catch (ex) {
	    XmlDigitalSigTool.logError("gFileUtil.test1() failed - ex: " + ex + "");
	}
    }
    {
	try  {
    	    // gFileUtil.test1();
	    var emptyDoc = gX509CertDBUtil.createDocument("http://abc.com/pqrst", "zyx");
	    XmlDigitalSigTool.dump("gX509CertDBUtil.createDocument():");
	    XmlDigitalSigTool.dump("gX509CertDBUtil.createDocument(): emptyDoc: " + gFileUtil.saveXMLDocToString(emptyDoc));
	} catch (ex) {
	    XmlDigitalSigTool.logError("gFileUtil.test1() failed - ex: " + ex + "");
	}
    }
    */

    gXmlParser = new DOMParser();
    gXmlSerializer = new XMLSerializer();

    gTabboxElem = document.getElementById('xmldsigffext.form.tabbox');
    gSignTabElem = document.getElementById('xmldsigffext.form.tab.sign');
    gVerifyTabElem = document.getElementById('xmldsigffext.form.tab.verify');

    gPreviewCmdButtonElem = document.getElementById('xmldsigffext.cmd.preview');
    gSignCmdButtonElem = document.getElementById('xmldsigffext.cmd.sign');
    gVerifyCmdButtonElem = document.getElementById('xmldsigffext.cmd.verify');

    try {
    	// xmldsigffext_initSignFormXULElems();

    	xmldsigffext_initVerifyFormXULElems();

	// xmldsigffext_displaySignTab();
	xmldsigffext_displayVerifyTab();

    } catch (ex) {
    	XmlDigitalSigTool.logError("xmldsigffext_initOnLoad(): ex - " + ex + "");
    }

    xmldsigffext_initParamsOnLoad();

    /**********************************************/
    /* TODO:  TEST CODE - remove after test phase */
    /**********************************************/
    var testMode = avxmldsig.dsigtool.PrefUtil.getPreference("xmldsigffext.test", false);
    if (testMode) {
    try {
        var tokenPassword = "abcd1234";
        var tokendb = Components.classes["@mozilla.org/security/pk11tokendb;1"].
                            getService(Components.interfaces.nsIPK11TokenDB);
        var token = tokendb.getInternalKeyToken();
        if (token) {
            token.checkPassword(tokenPassword);
            token.login(false);
        }
    } catch (ex) {}
    }

    XmlDigitalSigTool.logTrace("xmldsigffext_initOnLoad():................End.");
}


function xmldsigffext_initParamsOnLoad()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_initParamsOnLoad():................Start.");

    if ((!window.arguments) || (window.arguments.length == 0) || !window.arguments[0]) {
    	XmlDigitalSigTool.logTrace("xmldsigffext_initParamsOnLoad():................End(0).");
    	return false;
    }

    var dialogParams = window.arguments[0].QueryInterface(Components.interfaces.nsIDialogParamBlock);
    var pkiParams = dialogParams.QueryInterface(Components.interfaces.nsIPKIParamBlock);
    gParams = dialogParams;

    var strParamCnt = dialogParams.GetInt(1);
    if (strParamCnt <= 0) {
    	XmlDigitalSigTool.logTrace("xmldsigffext_initParamsOnLoad():................End(1).");
    	return false;
    }

    var inXMLFilePath = dialogParams.GetString(0);
    var outXMLFilePath = dialogParams.GetString(1);

    XmlDigitalSigTool.logDebug("xmldsigffext_initParamsOnLoad(): " + 
    		"inXMLFilePath: " + inXMLFilePath + " " + 
    		"outXMLFilePath: " + outXMLFilePath + " " + 
		"");

    var canonicalizationMethod = ""; 
    var credentialType = "";
    var signerX509CertList = [];
    var totSignerX509CertCnt = 0;
    var signerX509Cert = null;

    var certCnt = 0;
    try {
    	certCnt = dialogParams.GetInt(2);
    } catch (ex) { }
    if (certCnt > 0) {
    	var x509Cert = null;
	if (pkiParams) {
	    x509Cert = pkiParams.getISupportAtIndex( 0 + 1);
	}
	if (x509Cert) {
	    credentialType = "certificate";
	    signerX509Cert = x509Cert.QueryInterface(Components.interfaces.nsIX509Cert);
	    signerX509CertList.push(signerX509Cert);
	}
    }
    totSignerX509CertCnt = signerX509CertList.length;
    XmlDigitalSigTool.logDebug("xmldsigffext_initParamsOnLoad(): certCnt: " + certCnt + " signerX509Cert: " + (signerX509Cert ? signerX509Cert.nickname : "null") + ""); 
    
    if (inXMLFilePath && (inXMLFilePath != "")) {
    	gXmlDSigJsonParams.inputXmlDocFilePath = inXMLFilePath;
    }
    if (outXMLFilePath && (outXMLFilePath != "")) {
    	gXmlDSigJsonParams.outXmlDocFilePath = outXMLFilePath;
    }
    if (credentialType && (credentialType != "")) {
    	gXmlDSigJsonParams.credentialType = credentialType;
    }
    if (canonicalizationMethod && (canonicalizationMethod != "")) {
    	gXmlDSigJsonParams.canonicalizationMethod = canonicalizationMethod;
    }
    gXmlDSigJsonParams.totSignerX509CertCnt = totSignerX509CertCnt;
    gXmlDSigJsonParams.signerX509CertList = signerX509CertList;
    gXmlDSigJsonParams.signerX509Cert = signerX509Cert;

    xmldsigffext_doOnloadInitSignParam(inXMLFilePath, outXMLFilePath, signerX509Cert);

    XmlDigitalSigTool.logTrace("xmldsigffext_initParamsOnLoad():................End.");
    return true;
}

function xmldsigffext_showSignForm()
{
    // XmlDigitalSigTool.logTrace("xmldsigffext_showSignForm():................Start.");

    gPreviewCmdButtonElem.hidden = false;
    gSignCmdButtonElem.hidden = false;
    gVerifyCmdButtonElem.hidden = true;

    // XmlDigitalSigTool.logTrace("xmldsigffext_showSignForm():................End.");
}

function xmldsigffext_showVerifyForm()
{
    // XmlDigitalSigTool.logTrace("xmldsigffext_showVerifyForm():................Start.");

    gPreviewCmdButtonElem.hidden = true;
    gSignCmdButtonElem.hidden = true;
    gVerifyCmdButtonElem.hidden = false;

    // XmlDigitalSigTool.logTrace("xmldsigffext_showVerifyForm():................End.");
}

function xmldsigffext_displayTab(tabElemId)
{
    XmlDigitalSigTool.logTrace("xmldsigffext_displayTab():................Start.");

    var tabElem = document.getElementById(tabElemId);
    gTabboxElem.selectedTab = tabElem;

    XmlDigitalSigTool.logTrace("xmldsigffext_displayTab():................End.");
}

function xmldsigffext_displaySignTab()
{
    // xmldsigffext_displayTab("xmldsigffext.form.tab.sign");
    xmldsigffext_showSignForm();
}

function xmldsigffext_displayVerifyTab()
{
    // XmlDigitalSigTool.logTrace("xmldsigffext_displayVerifyTab():................Start.");

    // xmldsigffext_displayTab("xmldsigffext.form.tab.verify");
    xmldsigffext_showVerifyForm();

    // XmlDigitalSigTool.logTrace("xmldsigffext_displayVerifyTab():................End.");
}

function xmldsigffext_enableForVerify()
{
    xmldsigffext_showVerifyForm();
}


function xmldsigffext_enableForSign()
{
    xmldsigffext_showSignForm();
}

function xmldsigffext_enableForResign()
{
    xmldsigffext_showSignForm();
}

function xmldsigffext_cmdVerify()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_cmdVerify():..................Start.");
    if (gParams) {
	// Falure by default
        gParams.SetInt(0, 0);
    }
    if (!gXmlDSigTool) {
    	alert(XPILoadErrorMessage1 + "\n" + XPILoadErrorMessage2);
    	// window.close();
	return;
    }

    var closeButtonElem = document.getElementById('xmldsigffext.cmd.close');
    var cancelButtonElem = document.getElementById('xmldsigffext.cmd.cancel');

    var isPassed = false;
    try {
    	// isPassed = xmldsigffext_verifySignedXMLDoc();
    	isPassed = xmldsigffext_verifySignedXMLFile();
    } catch (ex) {
    	alert("xmldsigffext_cmdVerify(): failed to verify signed XML file - ex : " + ex + "");
    	XmlDigitalSigTool.logError("xmldsigffext_cmdVerify(): failed to verify signed XML file - ex : " + ex + "");
	return;
    }
    if (isPassed) {
    	closeButtonElem.hidden = false;
    	cancelButtonElem.hidden = true;
    	if (gParams) {
            gParams.SetInt(1, 1); // verification passes
	}
    	XmlDigitalSigTool.logDebug("xmldsigffext_cmdVerify(): successfully verified the signed XML document.");
    	alert("Successfully verified the signed XML document.");
    }
    else {
    	XmlDigitalSigTool.logDebug("xmldsigffext_cmdVerify(): failed to verify the signed XML document.");
    	alert("Failed to verify the signed XML document.");
    	if (gParams) {
            gParams.SetInt(1, 0); // verification failed
	}
    }

    if (gParams) {
	// Set success 
        gParams.SetInt(0, 1);
    	window.close();
    }

    XmlDigitalSigTool.logTrace("xmldsigffext_cmdVerify():..................End.");
}

function xmldsigffext_cmdPreview()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_cmdPreview():..................Start.");

    xmldsigffext_previewXMLDoc();

    XmlDigitalSigTool.logTrace("xmldsigffext_cmdPreview():..................End.");
}

function xmldsigffext_cmdSign()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_cmdSign():..................Start.");
    if (gParams) {
	// Failure by default
        gParams.SetInt(0, 0);
    }
    if (!gXmlDSigTool) {
    	alert(XPILoadErrorMessage1 + "\n" + XPILoadErrorMessage2);
    	// window.close();
	return;
    }

    var signedFile = null;
    try {
    	// signedFile = xmldsigffext_signXMLDocToFile();
    	signedFile = xmldsigffext_signXMLFile();
    } catch (ex) {
    	XmlDigitalSigTool.logError("xmldsigffext_cmdSign(): failed to sign XML document - ex : " + ex + "");
    	alert("xmldsigffext_cmdSign(): failed to sign XML document - ex : " + ex);
	return;
    }
    if (!signedFile) {
    	XmlDigitalSigTool.logError("xmldsigffext_cmdSign(): failed to create signed XML file");
    	alert("xmldsigffext_cmdSign(): failed to create signed XML file");
	return;
    }
    var closeButtonElem = document.getElementById('xmldsigffext.cmd.close');
    var cancelButtonElem = document.getElementById('xmldsigffext.cmd.cancel');

    closeButtonElem.hidden = false;
    cancelButtonElem.hidden = true;

    if (gParams) {
	// Set success 
        gParams.SetInt(0, 1);
    	window.close();
    }
    else {
    	XmlDigitalSigTool.logDebug("xmldsigffext_cmdSign(): Successfully created the signed XML document.");
    	alert("Successfully created the signed XML document.");
    }
    XmlDigitalSigTool.logTrace("xmldsigffext_cmdSign():..................End.");
}

function xmldsigffext_cmdClose()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_cmdClose():..................Start.");
    window.close();
}

function xmldsigffext_cmdCancel()
{
    XmlDigitalSigTool.logTrace("xmldsigffext_cmdCancel():..................Start.");
    if (gParams) {
        gParams.SetInt(0, 0);
    }
    window.close();
}



/*
function xmldsigffext_handleEnableEncryptSignature(enableEncryptElem)
{
    XmlDigitalSigTool.logTrace("xmldsigffext_handleEnableEncryptSignature():..................Start.");

    var encryptCertRowElem = document.getElementById('xmldsigffext.form.encrypt.cert.row');
    encryptCertRowElem.hidden = !enableEncryptElem.checked;

    XmlDigitalSigTool.logTrace("xmldsigffext_handleEnableEncryptSignature():..................End.");
}

function xmldsigffext_EncryptNickNameMenuListChanged(menuListElem, selectedItemElem)
{
    XmlDigitalSigTool.logTrace("xmldsigffext_EncryptNickNameMenuListChanged():..................Start/End.");
}
*/



