/* @(#) $Id: XmlMultiDigitalSigUtil.js,v 1.10 2012/12/05 19:02:05 subrata Exp $ */

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is  Avaya Labs Research, Avaya Inc. Code.
 *
 * The Initial Developer of the Original Code is
 *     Subrata Mazumdar, Subrata Mazumdar, Avaya Labs Research, Avaya Inc.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Subrata Mazumdar (mazum@avaya.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/*
// JSON format for DSig params
var gXmlMultiSigDocParams = {
    "inputXmlDocFilePath"	: null,
    "inputXmlDocFileURI"	: null,
    "inputXmlDoc"		: null,
    "contentElementURI"		: "",
    "credentialType"		: "certificate",
    "canonicalizationMethod"	: "http://www.w3.org/TR/2001/REC-xml-c14n-20010315",
    "totSignerX509CertCnt"	: 1,
    "signerX509CertList"	: [],
};
*/

var XmlMultiDSigUtil = {

    mXmlParser : null,
    mXmlSerializer : null,
    mAlreadyInit : false,

    /* nsIX509Cert	  */ mX509CertDB : null,
    /* alrIX509CertDBUtil */ mX509CertDBUtil : null,
    /* alrIXPXmlDSigTool  */ mXmlDSigTool : null,


    mMaxLogLevel : 2,
    log : function(level, msg)
    {
	if (level > this.mMaxLogLevel) {
	    return;
	}
	dump(msg + "\n");
    },

    dump : function(msg)
    {
	this.log(1, msg);
    },
    logError : function(msg)
    {
	this.log(2, msg);
    },
    logTrace : function(msg)
    {
	this.log(4, msg);
    },
    logDebug : function(msg)
    {
	this.log(8, msg);
    },

    dialogIntParamPositions : {
	"strParamCnt" : 1,
	"certCnt" : 2
    },
    dialogStrParamPositions : {
	"inputXmlDocFilePath"	: 0,
	"outXmlDocFilePath"	: 1,
	"contentElementURI"	: 2,
	"signatureType"		: 3,
	"credentialType"	: 4,
	"canonicalizationMethod" : 5,
	"totSignerX509CertCnt"	: 6
    },

    /* nsIFile */
    getUserProfileDir : function ()
    {
	// First get the directory service and query interface it to
	// nsIProperties
	var dirService = Components.classes['@mozilla.org/file/directory_service;1']
			.getService(Components.interfaces.nsIProperties);
	var profileDir = dirService.get("ProfD", Components.interfaces.nsIFile);
	return profileDir;
    },

    /* nsIFile */
    getToolAddonDir : function ()
    {
	var profileDir = XmlMultiDSigUtil.getUserProfileDir();
	var toolAddonDir = profileDir.clone();
	toolAddonDir.append("extensions");
	toolAddonDir.append("{bcb4f322-a177-4ecd-9c80-835d96e1e595}");
	return toolAddonDir;
    },


    getTagertPlatformName : function ()
    {
	// XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.getTagertPlatformName(): userAgent: " +  navigator.userAgent + "");

	var isWin = (navigator.userAgent.indexOf("Win") != -1);
	var isMac = (navigator.userAgent.indexOf("Mac") != -1);
	var isLinux = (navigator.userAgent.indexOf("Linux") != -1);
	var isLinux64 = false;
	if (isLinux) {
		isLinux64 = (navigator.userAgent.indexOf("x86_64") != -1);
	}

	var targetPlatformName = "";
	if (isWin) {
		targetPlatformName = "WINNT_x86-msvc";
	}
	else if (isLinux) {
		if (isLinux64) {
		    targetPlatformName = "Linux_x86_64-gcc3";
	}
	else {
		    targetPlatformName = "Linux_x86-gcc3";
	}
	}
	else if (isMac) {
		targetPlatformName = "Darwin_x86-gcc3";
	}
	return targetPlatformName;
    },

    /* nsIFile */
    getToolPlatformDepCompDir : function ()
    {
	var toolDepcompDir = XmlMultiDSigUtil.getToolAddonDir();
	var targetPlatformName = XmlMultiDSigUtil.getTagertPlatformName();
	if (targetPlatformName == "") {
		return toolDepcompDir;
	}
	toolDepcompDir.append("platform");
	toolDepcompDir.append(targetPlatformName);
	toolDepcompDir.append("depcomps");
	return toolDepcompDir;
    },

    isSupportedOS : function ()
    {
	var winNavigator = window.navigator;
	var userAgent = window.navigator.userAgent;

	/*
	var isWin = (userAgent.indexOf("Win") != -1);
	var isLinux = (userAgent.indexOf("Linux") != -1);
	// var isMac = (userAgent.indexOf("Mac") != -1);
	if (!(isWin || isLinux)) {
		alert("ERROR: WRONG BROWSER USER AGENT (" + userAgent + ")\n" +
		"This tool is only supported on Linux_x86-gcc3 and Windows_x86-msvc platforms.");
		return false;
	}
	*/

	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.isSupportedOS(): userAgent: " + userAgent + "");
	/*
	if (window.navigator.onLine) {
	// Touch base - experimental
	var remoteURL = "http://www.research.avayalabs.com/gcm/usa/en-us/people/all/subratamazumdar.htm";
	var httpMethod = "HEAD";
	try {
	    var req = new XMLHttpRequest(); // Request
	    req.open(httpMethod, remoteURL, false);  // false ==> synchronous
	    req.send("");
	    XmlMultiDSigUtil.dump("XmlMultiDSigUtil.isSupportedOS(): HTTP HEAD - req.status: " + req.status + "\n");
	    if ((req.status / 100) != 2 ) {
		XmlMultiDSigUtil.dump("XmlMultiDSigUtil.isSupportedOS(): HTTP HEAD failed - req.status: " + req.status + "\n");
	    }
	} catch (ex) {}
	}
	*/
	return true;
    },

    checkForXPCOMServices : function ()
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.checkForXPCOMServices():................Start.");
	const XPILoadErrorMessage1 = "Failed to load shared librararies and/or register XPCOM componenets for XML Digital Signature tool ."
	const XPILoadErrorMessage2 = "Make sure that DLL (shared libraries) for Apache Xerces-C++ is in your PATH (LD_LIBRARY_PATH) environment variable.";


	var result = false;
	var errmsg1 = XPILoadErrorMessage1;
	var errmsg2 = XPILoadErrorMessage2;
	var platformDepCompDir = XmlMultiDSigUtil.getToolPlatformDepCompDir();
	var platformDepCompDirPath = platformDepCompDir.path;
	var errmsg3 = "";
	var isMac = (navigator.userAgent.indexOf("Mac") != -1);
	if (isMac) {
	    errmsg3 = "Adding the follwoing path to your DYLD_LIBRARY_PATH : " + platformDepCompDirPath  + " before installing the add-on may help.";
	}

	if (!Components.classes["@avaya.com/pkm/x509certdbutil;1"]
		|| !Components.classes["@avaya.com/pkm/xmldsigtool;1"]
		) {
		XmlMultiDSigUtil.logError(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "");
		alert(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
		throw errmsg1;
	}
		
	if (!Components.interfaces.nsIX509CertDB
		|| !Components.interfaces.alrIX509CertDBUtil
		|| !Components.interfaces.alrIXPXmlDSigTool 
		) {
		XmlMultiDSigUtil.logError(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "");
		alert(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
		throw errmsg1;
	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.checkForXPCOMServices():................End.");
	return true;
    },

    initXPCOMServices : function ()
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.initXPCOMServices():................Start.");

	if (XmlMultiDSigUtil.mAlreadyInit) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.initXPCOMServices():................End(0).");
	    return;
	}
	XmlMultiDSigUtil.mAlreadyInit = true;

	XmlMultiDSigUtil.isSupportedOS();
	XmlMultiDSigUtil.checkForXPCOMServices();
	try {
	    XmlMultiDSigUtil.mX509CertDB = Components.classes["@mozilla.org/security/x509certdb;1"]
				.getService(Components.interfaces.nsIX509CertDB);
	    XmlMultiDSigUtil.mX509CertDBUtil = Components.classes["@avaya.com/pkm/x509certdbutil;1"]
				.getService(Components.interfaces.alrIX509CertDBUtil);
	    XmlMultiDSigUtil.mXmlDSigTool = Components.classes["@avaya.com/pkm/xmldsigtool;1"]
				.getService(Components.interfaces.alrIXPXmlDSigTool);
	    XmlMultiDSigUtil.mIOService = Components.classes["@mozilla.org/network/io-service;1"]
				.getService(Components.interfaces.nsIIOService);
	} catch (ex) {
	    alert("XmlMultiDSigUtil.initXPCOMServices(): failed to create instance of alrIXPXmlDSigTool - ex: " + ex + "");
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.initXPCOMServices(): failed to create instance of alrIXPXmlDSigTool - ex: " + ex + "");
	    XmlMultiDSigUtil.logError(XPILoadErrorMessage2 + "");
	    throw ex;
	}
	if (XmlMultiDSigUtil.mXmlDSigTool == null) {
	    alert("XmlMultiDSigUtil.initXPCOMServices(): failed to create instance of alrIXPXmlDSigTool.\n" + XPILoadErrorMessage2);
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.initXPCOMServices(): failed to create instance of alrIXPXmlDSigTool." + XPILoadErrorMessage2);
	    throw XPILoadErrorMessage2;
	}

	XmlMultiDSigUtil.mXmlParser = new DOMParser();
	XmlMultiDSigUtil.mXmlSerializer = new XMLSerializer();

	/**********************************************/
	/* TODO:  TEST CODE - remove after test phase */
	/**********************************************/
	/*
	var testMode = avxmldsig.dsigtool.PrefUtil.getPreference("xmldsigffext.test", false);
	if (testMode) {
	try {
	    var tokenPassword = "abcd1234";
	    var tokendb = Components.classes["@mozilla.org/security/pk11tokendb;1"].
				getService(Components.interfaces.nsIPK11TokenDB);
	    var token = tokendb.getInternalKeyToken();
	    if (token) {
		token.checkPassword(tokenPassword);
		token.login(false);
	    }
	} catch (ex) {}
	}
	*/

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.initXPCOMServices():................End.");
    },

    readXMLDocFromString : function (aXmlDocDataStr)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromString():................Start.");

	var /* nsIDOMDocument */ xmlDoc = null;
	try {
	    xmlDoc = XmlMultiDSigUtil.mXmlParser.parseFromString(aXmlDocDataStr, "text/xml");
	} catch (ex) {
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromString(): XmlMultiDSigUtil.mXmlParser.parseFromString() failed - ex: " + ex);
	    // return null;
	    throw ex;
	}
	var docElemTag = xmlDoc.documentElement;
	if ((docElemTag.tagName == "parserError")
	    || (docElemTag.namespaceURI == "http://www.mozilla.org/newlayout/xml/parsererror.xml")){
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromString(): XmlMultiDSigUtil.mXmlParser.parseFromString() failed - parserError");
	    // return null;
	    throw "parserError";
	}

	/*
	try {
	    xmlDoc = XmlMultiDSigUtil.mXmlDSigTool.readXMLDocFromString(aXmlDocDataStr);
	} catch (ex) {
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromString(): XmlMultiDSigUtil.mXmlDSigTool.readXMLDocFromString() failed - ex: " + ex + "");
	    throw ex;
	}
	*/

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromString():................End.");
	return xmlDoc;
    },

    readDataFromURI : function (aXmlDocURI)
    {
	XmlMultiDSigUtil.logTrace( "XmlMultiDSigUtil.readDataFromURI():................Start.");

	var ioService = Components.classes["@mozilla.org/network/io-service;1"]
				  .getService(Components.interfaces.nsIIOService);
	    
	// get a channel for that nsIURI
	var channel = ioService.newChannelFromURI(aXmlDocURI);
	if (!channel) {
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readDataFromURL(): ioService.newChannelFromURI() failed.");
	    return null;
	}
    
	var channelIS = null;
	/*
	var jarChannel = channel.QueryInterface(Components.interfaces.nsIJARChannel);
	if (jarChannel) {
	    channelIS = jarChannel.open();
	}
	else {
	    channelIS = channel.open();
	}
	*/
	channelIS = channel.open();
	if (!channelIS) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readDataFromURI(): channel.open() failed.");
	    return null;
	}
    
	var siStream = Components.classes["@mozilla.org/scriptableinputstream;1"]
			.createInstance(Components.interfaces.nsIScriptableInputStream);
	siStream.init(channelIS);

	var dataCnt = 0;
	try {
	    dataCnt = siStream.available();
	} catch (ex) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readDataFromURL(): siStream.available() failed - ex: " + ex);
	    throw ex;
	}
	if (dataCnt == 0) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readDataFromURL(): siStream.available() failed.");
	    return null;
	}

	var urlDataStr = "";
	var str = siStream.read(-1);
	while (str.length > 0) {
	    urlDataStr += str;
	    str = siStream.read(-1);
	}
    
	siStream.close();
	channelIS.close();
    
	XmlMultiDSigUtil.logTrace( "XmlMultiDSigUtil.readDataFromURI():................End.");
	return urlDataStr;
    },

    readDataFromURISpec : function (aXmlDocURISpec)
    {
	var ioService = Components.classes["@mozilla.org/network/io-service;1"]
				  .getService(Components.interfaces.nsIIOService);
	// create an nsIURI
	var dataURI = XmlMultiDSigUtil.mIOService.newURI(aXmlDocURISpec, null, null);

	var dataStr =  XmlMultiDSigUtil.readDataFromURI(dataURI);
	return dataStr;
    },

    readXMLDocFromURI : function (aXmlDocURI)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromURI():................Start.");

	var xmlDataStr =  XmlMultiDSigUtil.readDataFromURI(aXmlDocURI);
	if (!xmlDataStr) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromURI():................End(1).");
	    return null;
	}
	var xmlDoc = XmlMultiDSigUtil.readXMLDocFromString(xmlDataStr);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromURI():................End.");
	return xmlDoc;
    },

    readXMLDocFromURISpec : function (aXmlDocURISpec)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromURI():................Start.");

	var xmlDataStr =  XmlMultiDSigUtil.readDataFromURISpec(aXmlDocURISpec);
	if (!xmlDataStr) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromURI():................End(1).");
	    return null;
	}
	var xmlDoc = XmlMultiDSigUtil.readXMLDocFromString(xmlDataStr);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromURI():................End.");
	return xmlDoc;
    },

    XYZ_readXMLDocFromURI : function (aXmlDocURI)
    {
    	var /* nsIDOMDocument */ xmlDoc = null;
    	try {
    	    xmlDoc = XmlMultiDSigUtil.mXmlDSigTool.readXMLDocFromURIPath(aXmlDocURI);
    	} catch (ex) {
    	    alert("Not a valid XML file - parsing error.");
    	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromURI(): Not a valid XML file - parsing error.");
    	    throw ex;
    	}
    	if (!xmlDoc) {
    	    alert("Not a valid XML file.");
    	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromURI(): Not a valid XML file.");
    	    return;
    	}
    	if (xmlDoc.documentElement.tagName == "parsererror") {
    	    alert("Parsing of  XML file failed.");
    	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromURI(): Parsing of  XML file failed.");
    	    return;
    	}
	return xmlDoc;
    },

    readXMLDocFromFile : function (aXmlDocFile)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromFile():................Start.");

	var uri = XmlMultiDSigUtil.mIOService.newFileURI(aXmlDocFile);
	// var xmlDoc =  XmlMultiDSigUtil.readXMLDocFromURI(uri.spec);
	var xmlDoc =  XmlMultiDSigUtil.readXMLDocFromURI(uri);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromFile():................End.");
	return xmlDoc;
    },
    
    XYZ_readXMLDocFromFile : function (aXmlDocFile)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromFile():................Start.");

    	var /* nsIDOMDocument */ xmlDoc = null;
    	try {
    	    xmlDoc = XmlMultiDSigUtil.mXmlDSigTool.readXMLDocFromFile(aXmlDocFile);
    	} catch (ex) {
    	    alert("Not a valid XML file - parsing error.");
    	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromFile(): Not a valid XML file - parsing error.");
    	    throw ex;
    	}
    	if (!xmlDoc) {
    	    alert("Not a valid XML file.");
    	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromFile(): Not a valid XML file.");
    	    return;
    	}
    	if (xmlDoc.documentElement.tagName == "parsererror") {
    	    alert("Parsing of  XML file failed.");
    	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromFile(): Parsing of  XML file failed.");
    	    return;
    	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromFile():................End.");
	return xmlDoc;
    },

    readXMLDocFromFilePath : function (aXmlDocFilePath)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromFilePath():................Start.");
	var /* nsIDOMDocument */ xmlDoc = null;

	try {
	    var /* nsIFile */ xmlDocFile = Components.classes["@mozilla.org/file/local;1"]
	                                  .createInstance(Components.interfaces.nsILocalFile);
    	    xmlDocFile.initWithPath(aXmlDocFilePath);
	    xmlDoc = XmlMultiDSigUtil.readXMLDocFromFile(xmlDocFile);
	} catch (ex) {
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.readXMLDocFromFilePath(): readXMLDocFromFile() failed - ex: " + ex + "");
	    throw ex;
	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.readXMLDocFromFilePath():................End.");
	return xmlDoc;
    },

    getDocElementById : function (aXmlDoc, aElemId)
    {
	// XmlMultiDSigUtil.mMaxLogLevel = 9;
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getDocElementById():................Start.");

	var retElem = aXmlDoc.getElementById(aElemId);
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.getDocElementById(): retElem: " + retElem);
	if (retElem) {
	    return retElem;
	}

	var docElemList = aXmlDoc.getElementsByTagName("*");
	var idAttrNames = ["id", "Id", "ID"];
	for (var i = 0; i < docElemList.length; i++) {
	    var docElem = docElemList.item(i);
	    // XmlMultiDSigUtil.dump("getDocElementById(): docElem:\n" + XmlMultiDSigUtil.mXmlSerializer.serializeToString(docElem) + "\n");
	    var idVal = null;
	    for (var j = 0; (j < idAttrNames.length) && !idVal; j++) {
		var idAttrName = idAttrNames[j];
	    	idVal = docElem.getAttribute(idAttrName);
	    	// XmlMultiDSigUtil.dump("getDocElementById(): docElem.tagName: " + docElem.tagName + " idName: " + idAttrName + " idVal: " + idVal + "\n");
	    }
	    if (!idVal || (idVal != aElemId)) {
	    	continue;
	    }
	    
	    retElem = docElem;
	    break;
	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getDocElementById():................End.");
	return retElem;
    },

    getChildElementsByTagName : function(aDocElem, aTagName)
    {
	XmlMultiDSigUtil.logTrace("getChildElementsByTagName():..........................Start.");
	
	var childElemList = [];

	if (!aDocElem) {
	    return null;
	}

	if (aDocElem.childElementCount == 0) {
	    return null;
	}

	for (var i = 0; i < childElemList.length; i++) {
	    var childElem = childElemList.item(i);
	    if (childElem.nodeName != aTagName) {
		continue;
	    }
	    childElemList.push(childElem);
	}

	XmlMultiDSigUtil.logTrace("getChildElementsByTagName():..........................End.");
	return childElemList;
    },

    getFirstChildElementByTagName : function (aDocElem, aTagName)
    {
	XmlMultiDSigUtil.logTrace("getFirstChildElementByTagName():..........................Start.");
	if (!aDocElem || !aDocElem.hasChildNodes()) {
	    return null;
	}
	if (aDocElem.childElementCount == 0) {
	    return null;
	}

	var foundChildElem = null;
	var childElem = aDocElem.firstElementChild; 
	while (childElem) {
	    if (childElem.localName == aTagName) {
		foundChildElem = childElem;
		break;
	    }
	    childElem = childElem.nextElementSibling;
	}

	/*
	var childElemList = aDocElem.children;
	for (var i = 0; i < childElemList.length; i++) {
	    var childElem = childElemList.item(i);
	    if (childElem.localName == aTagName) {
		foundChildElem = childElem;
		break;
	    }
	}
	*/

	XmlMultiDSigUtil.logTrace("getFirstChildElementByTagName():..........................End(ERROR).");
	return foundChildElem;
    },
    getLastChildElementByTagName : function (aDocElem, aTagName)
    {
	XmlMultiDSigUtil.logTrace("getLastChildElementByTagName():..........................Start.");
	if (!aDocElem || !aDocElem.hasChildNodes()) {
	    return null;
	}
	if (aDocElem.childElementCount == 0) {
	    return null;
	}

	var foundChildElem = null;
	var childElem = aDocElem.lastElementChild; 
	while (childElem) {
	    if (childElem.localName == aTagName) {
		foundChildElem = childElem;
		break;
	    }
	    childElem = childElem.previousElementSibling;
	}

	/*
	var childElemList = aDocElem.children;
	for (var i = (childElemList.length -1); i >= 0; i--) {
	    var childElem = childElemList.item(i);
	    if (childElem.localName == aTagName) {
		foundChildElem = childElem;
		break;
	    }
	}
	*/

	XmlMultiDSigUtil.logTrace("getLastChildElementByTagName():..........................End(ERROR).");
	return foundChildElem;
    },
    containsAnyElement : function (aDocElem)
    {
	var retVal = false;
	if (aDocElem && aDocElem.firstChildElement) {
	    retVal = true;
	}
	return retVal;
    },

    getElementTextValue : function (aDocElem)
    {
	if (!aDocElem || !aDocElem.hasChildNodes()) {
	    return null;
	}
	var textValue = XmlMultiDSigUtil.trimString(aDocElem.firstChild.nodeValue);
	return textValue;
    },

    setElementTextValue : function (aDocElem, aDocElemValue)
    {
	XmlMultiDSigUtil.logTrace("setElementTextValue():..........................Start");
	if (!aDocElem) {
	    return;
	}

	if (!aDocElem.hasChildNodes()) {
	    var textNode = aDocElem.ownerDocument.createTextNode("");
	    textNode = aDocElem.appendChild(textNode);
	}

	aDocElem.firstChild.nodeValue = aDocElemValue;

	XmlMultiDSigUtil.logTrace("setElementTextValue():..........................End");
    },

    createKeyInfoByX509Cert : function (/* nsIX509Cert */ aX509cert, /* nsIDOMElement */ aKeyInfoParentElem, aKeyInfoParamOptions)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.createKeyInfoByX509Cert():................Start.");
	var keyInfoElem = null;
    	/*
    	const unsigned long KEYINFO_KeyName			= 1 << 1;
    	const unsigned long KEYINFO_KeyValue 		= 1 << 2;
    	// const unsigned long KEYINFO_SHA1FingerPrint		= 1 << 3;
    	const unsigned long KEYINFO_X509Data_Certificate	= 1 << 3;
    	const unsigned long KEYINFO_X509Data_SubjectName	= 1 << 4;
    	const unsigned long KEYINFO_X509Data_IssuerSerial	= 1 << 5;
    	const unsigned long KEYINFO_X509Data_SHA1FingerPrint	= 1 << 6;
    	const unsigned long KEYINFO_X509Data_KeyIdentifier	= 1 << 6;
    	const unsigned long KEYINFO_X509Data_SKI		= 1 << 7;
    	const unsigned long KEYINFO_EncryptedKey 		= 1 << 8;
    	*/
	var keyInfoParamOptions = aKeyInfoParamOptions;
	if (!keyInfoParamOptions) {
	    keyInfoParamOptions = Components.interfaces.alrIX509CertDBUtil.KEYINFO_X509Data_Certificate ||
	    			Components.interfaces.alrIX509CertDBUtil.KEYINFO_X509Data_SubjectName;
	}
	try {
	    keyInfoElem = XmlMultiDSigUtil.mX509CertDBUtil.exportX509CertToKeyInfo(
	    			aX509cert,
				aKeyInfoParentElem,
				keyInfoParamOptions
				);
	} catch (ex) {
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.createKeyInfoByX509Cert(): XmlMultiDSigUtil.mX509CertDBUtil.exportX509CertToKeyInfo() failed - ex: " + ex);
	    throw ex;
	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.createKeyInfoByX509Cert():................End.");
	return keyInfoElem;
    },

    getDefaultSigTemplate : function (aXmlMultiSigDocParams)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getDefaultSigTemplate():................Start.");

	var xmlMultiSigDocParams = aXmlMultiSigDocParams || {};

	var referenceURI = "";
	var contentElementURI = "";
	if (xmlMultiSigDocParams.contentElementURI && (xmlMultiSigDocParams.contentElementURI != "")) {
	    contentElementURI = xmlMultiSigDocParams.contentElementURI;
	}
	referenceURI = contentElementURI;

	var envelopedTransformAlg = "http://www.w3.org/2000/09/xmldsig#enveloped-signature";
	var detatchedTransformAlg = "";
	var envelopingTransformAlg = "";
	var transformAlg = "";

	var SigRefAttrStr = "";
	var SigRefTxfmElemsStr = "";
	var EnvelopedTxfmElemsStr = "";
	var SigObjectElemStr = "";
	switch(xmlMultiSigDocParams.signatureType) {
	    case "ENVELOPING_SIGNATURE" :
		var envelopingObjId = "ObjectId_" + (new Date()).getTime();
		var refId = "Reference_" + (new Date()).getTime();

		var refType = "http://www.w3.org/2000/09/xmldsig#Object";
		referenceURI = "#" + envelopingObjId;
		SigRefAttrStr = "Id=\"" + refId + "\"" + " " + 
					"URI=\"" + referenceURI + "\"" + " " +
					"Type=\"" + refType + "\"";

		var EnvelopingObjectElemStr = 
"<ds:Object Id=\"" + envelopingObjId + "\">\n" +
"</ds:Object>\n" +
"";
		SigObjectElemStr = EnvelopingObjectElemStr;
		xmlMultiSigDocParams.envelopingObjectId = envelopingObjId;
		break;
	    case "DETACHED_SIGNATURE" :
		transformAlg = detatchedTransformAlg;
		SigRefAttrStr = "URI=\"" + referenceURI + "\"";
		break;
	    case "ENVELOPED_SIGNATURE" :
	    default:
		SigRefAttrStr = "URI=\"" + referenceURI + "\"";
		EnvelopedTxfmElemsStr = 
"<ds:Transforms>\n" +
"<ds:Transform Algorithm=\"" + envelopedTransformAlg + "\"/>\n" +
"</ds:Transforms>\n" +
"";
	    	SigRefTxfmElemsStr = EnvelopedTxfmElemsStr;
		transformAlg = envelopedTransformAlg;
		break;
	}

	var canonicalizationMethod = "http://www.w3.org/TR/2001/REC-xml-c14n-20010315";
	if (xmlMultiSigDocParams.canonicalizationMethod != "") {
	    canonicalizationMethod = xmlMultiSigDocParams.canonicalizationMethod;
	}

	var signatureMethod = "http://www.w3.org/2000/09/xmldsig#rsa-sha1";

	var credentialType = "certificate";
	if (xmlMultiSigDocParams.credentialType != "") {
	    credentialType = xmlMultiSigDocParams.credentialType;
	}

	var KeyInfoX509DataStr = "";
	/*
	if (credentialType == "certificate") {
	    KeyInfoX509DataStr = "<ds:X509Data><ds:X509Certificate/><ds:X509SubjectName/></ds:X509Data>\n";
	}
	else {
	    KeyInfoX509DataStr = "<ds:KeyName/>\n";
	}
	*/


	var EmptySignatureDoc =
"<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" +
"<ds:Signature xmlns:ds=\"http://www.w3.org/2000/09/xmldsig#\">\n" + 
"<ds:SignedInfo>\n" + 
"<ds:CanonicalizationMethod Algorithm=\"" + canonicalizationMethod + "\"/>\n" +
"<ds:SignatureMethod Algorithm=\"" + signatureMethod + "\"/>\n" +
"<ds:Reference " + SigRefAttrStr + ">\n" + 
SigRefTxfmElemsStr + 
"<ds:DigestMethod Algorithm=\"http://www.w3.org/2000/09/xmldsig#sha1\"/>\n" + 
"<ds:DigestValue/>\n" +
"</ds:Reference>\n" + 
"</ds:SignedInfo>\n" + 
"<ds:SignatureValue/>\n" +
"<ds:KeyInfo>\n"+
KeyInfoX509DataStr + 
"</ds:KeyInfo>\n"+
SigObjectElemStr + 
"</ds:Signature>\n" +
"";
	// var sigXmlDocStr = EmptySignatureDocBeginStr + TxfmElemsStr + EmptySignatureDocEndStr;
	var sigXmlDocStr = EmptySignatureDoc;

	var signatureElem = null;
	var sigXmlDoc = null; 
	try {
	    sigXmlDoc = XmlMultiDSigUtil.readXMLDocFromString(sigXmlDocStr);
	    signatureElem = sigXmlDoc.documentElement.cloneNode(true);
	    // signatureElem = sigXmlDoc.removeChild(sigXmlDoc.documentElement);
	} catch(ex) {
	    sigXmlDoc = null;
	    throw ex;
	}
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getDefaultSigTemplate():................30.");

	if (!signatureElem) {
	    return null;
	}

	// TODO: Add Replace KeyInfo element with KeyInfo for X.509 Cert if a cert is provided.

	var signerX509Cert = null;
	if (xmlMultiSigDocParams.signerX509CertList.length > 0) {
	    signerX509Cert = xmlMultiSigDocParams.signerX509CertList[0];
	}
	if (signerX509Cert) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getDefaultSigTemplate():................40.");
	    var sigKeyInfoElem = XmlMultiDSigUtil.getFirstChildElementByTagName(signatureElem, "KeyInfo");
	    if (sigKeyInfoElem) {
	    	signatureElem.removeChild(sigKeyInfoElem);
	    }
	    var keyInfoParamOptions = xmlMultiSigDocParams.keyInfoParamOptions;
	    if (!keyInfoParamOptions) {
	    	keyInfoParamOptions = Components.interfaces.alrIX509CertDBUtil.KEYINFO_X509Data_Certificate ||
	    				Components.interfaces.alrIX509CertDBUtil.KEYINFO_X509Data_SubjectName;
	    }
	    var keyInfoElem = XmlMultiDSigUtil.createKeyInfoByX509Cert(
	    				signerX509Cert,
					signatureElem,
					xmlMultiSigDocParams.keyInfoParamOptions
					);
	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getDefaultSigTemplate():................End.");
	return signatureElem;
    },

    getSignatureElem : function (aXmlMultiSigDocParams)
    {
    	return XmlMultiDSigUtil.getDefaultSigTemplate(aXmlMultiSigDocParams);
    },

    addSignatureElemToXmlDoc : function (aXmlDoc, aXmlMultiSigDocParams)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................Start.");

	var xmlMultiSigDocParams = aXmlMultiSigDocParams || {};
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): xmlMultiSigDocParams.contentElementURI: " + xmlMultiSigDocParams.contentElementURI);
    	var signatureElem = XmlMultiDSigUtil.getSignatureElem(aXmlMultiSigDocParams);

	var outXmlDoc = aXmlDoc.cloneNode(true);
	signatureElem = outXmlDoc.importNode(signatureElem, true);

	var sigContentElem = null;
	var sigContentElemId = xmlMultiSigDocParams.contentElementURI;
	if (xmlMultiSigDocParams.contentElementURI || (xmlMultiSigDocParams.contentElementURI != "")) {
	    sigContentElemId = sigContentElemId.substring(1); // Remove the '#' chararcter if present
	}
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): sigContentElemId: " + sigContentElemId);

	var inSigContentElem = null;
	if (!sigContentElemId  || (sigContentElemId == "")) {
	    sigContentElem = outXmlDoc.documentElement;
	    inSigContentElem = aXmlDoc.documentElement;
	}
	else {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................05.");
	    /*
	    sigContentElem = outXmlDoc.getElementById(sigContentElemId);
	    inSigContentElem = aXmlDoc.getElementById(sigContentElemId);
	    */
    	    sigContentElem = XmlMultiDSigUtil.getDocElementById(outXmlDoc, sigContentElemId);
    	    inSigContentElem = XmlMultiDSigUtil.getDocElementById(aXmlDoc, sigContentElemId);
	}
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): sigContentElem: " + sigContentElem);
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): inSigContentElem: " + inSigContentElem);
	if (!sigContentElem) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................End(0).");
	    return null;
	}
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................10.");

	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): xmlMultiSigDocParams.signatureType: " + xmlMultiSigDocParams.signatureType);
	switch(xmlMultiSigDocParams.signatureType) {
	    case "ENVELOPING_SIGNATURE" :
		XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................30.");
		var parentNode = outXmlDoc;
		if (!sigContentElem.parentNode.isSameNode(outXmlDoc)) {
		    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................32.");
		    parentNode = sigContentElem.parentNode;
		}
		sigContentElem = parentNode.replaceChild(signatureElem, sigContentElem);
		// XmlMultiDSigUtil.dump("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): outXmlDoc:\n" + XmlMultiDSigUtil.mXmlSerializer.serializeToString(outXmlDoc) + "\n");
		XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): sigObjElemId: " + aXmlMultiSigDocParams.envelopingObjectId);
		// var sigObjElem = outXmlDoc.getElementById(aXmlMultiSigDocParams.envelopingObjectId);
    	    	var sigObjElem = XmlMultiDSigUtil.getDocElementById(outXmlDoc, aXmlMultiSigDocParams.envelopingObjectId);
		if (!sigObjElem) {
		    outXmlDoc = null;
		    break;
		}
		XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): sigObjElem: " + sigObjElem);
		sigContentElem = sigObjElem.appendChild(sigContentElem);
		break;
	    case "DETACHED_SIGNATURE" :
		XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................40.");
		// Add the signature as sibling to the content element. Do we insert after the contentElem are 
		// at the end of nextElementSibling link.
		var containerElem = sigContentElem.parentNode;
		containerElem.insertBefore(signatureElem, sigContentElem.nextElementSibling);
		// containerElem.appendChild(sigContentElem);
		break;
	    case "ENVELOPED_SIGNATURE" :
	    default:
		XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................50.");
		// Add the signature as child to the content element.
		var containerElem = sigContentElem;
		containerElem.appendChild(signatureElem);
		break;
	}
	// XmlMultiDSigUtil.dump("XmlMultiDSigUtil.addSignatureElemToXmlDoc(): outXmlDoc:\n" + XmlMultiDSigUtil.mXmlSerializer.serializeToString(outXmlDoc) + "\n");

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.addSignatureElemToXmlDoc():................End.");
	return outXmlDoc;
    },

    getUnsignedXmlDoc : function (aInputXmlDoc)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getUnsignedXmlDoc():................Start.");

	var unsignedXmlDoc = aInputXmlDoc.cloneNode(true);
	// XmlMultiDSigUtil.dump("XmlMultiDSigUtil.getUnsignedXmlDoc(): inputXmlDoc:\n" + XmlMultiDSigUtil.mXmlSerializer.serializeToString(inputXmlDoc));

	var xmlDocSigElemList = unsignedXmlDoc.getElementsByTagName("ds:Signature");
	if (xmlDocSigElemList.length <= 0) {
	    xmlDocSigElemList = unsignedXmlDoc.getElementsByTagName("Signature");
	}

	if (xmlDocSigElemList.length <= 0) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getUnsignedXmlDoc():................End(0).");
	    return unsignedXmlDoc;
	}
	// Remove all Signature elements from the input XML doc to create the base XML doc for the signed XML doc.
	for (var j = (xmlDocSigElemList.length -1); j >= 0; j--) {
	    var sigElem = xmlDocSigElemList.item(j);
	    var sigParentElem = sigElem.parentNode;
	    sigParentElem.removeChild(sigElem);
	}

	// XmlMultiDSigUtil.dump("XmlMultiDSigUtil.getUnsignedXmlDoc(): unsignedXmlDoc:\n" + XmlMultiDSigUtil.mXmlSerializer.serializeToString(unsignedXmlDoc) + "\n");
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getUnsignedXmlDoc():................End.");
	return unsignedXmlDoc;
    },


    dumpMultiSigJsonParams : function (aMultiSigJsonParams)
    {
	var multiSigJsonParams = aMultiSigJsonParams || {};
	XmlMultiDSigUtil.dump("XmlMultiDSigUtil.dumpMultiSigJsonParams(): multiSigJsonParams : {\n");
	for (var paramName in multiSigJsonParams) {
	    XmlMultiDSigUtil.dump("    " + paramName + " : ");
	    if (paramName == "signerX509CertList") {
		dump ("{\n");
		for (var i = 0; i < multiSigJsonParams.signerX509CertList.length; i++) {
		    XmlMultiDSigUtil.dump("\t" + multiSigJsonParams.signerX509CertList[i].nickname + "\n");
		}
		dump ("    }\n");
	    }
	    else {
		XmlMultiDSigUtil.dump(multiSigJsonParams[paramName] + "\n");
	    }
	}
	XmlMultiDSigUtil.dump("}\n");
    },

    multiSigJsonParams2ToDailogParams : function (aMultiSigJsonParams, aMultiSigDailogParams)
    {
	// XmlMultiDSigUtil.mMaxLogLevel = 9;
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams():................Start.");

	var multiSigJsonParams = aMultiSigJsonParams || {};
	var dialogParams = aMultiSigDailogParams;
	var pkiParams = null;
	if (dialogParams) {
	    try {
		pkiParams = dialogParams.QueryInterface(Components.interfaces.nsIPKIParamBlock);
	    } catch (ex) {}
	}

	var str_argc = 0;
	var int_argc = 0;
	for (var paramName in multiSigJsonParams) {
	    var paramValue = multiSigJsonParams[paramName];

	    var argPos = XmlMultiDSigUtil.dialogStrParamPositions[paramName];
	    XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams(): paramName: " + paramName + " argPos: " + argPos + " paramValue: " + paramValue + "");
	    if ((argPos == 'undefined') || (argPos == null)) {
		continue;
	    }
	    str_argc++;

	    if (!paramValue) {
		continue;
	    }
	    if (typeof paramValue != "string") {
		continue;
	    }
	    dialogParams.SetString(argPos, paramValue);
	}
	dialogParams.SetInt(XmlMultiDSigUtil.dialogIntParamPositions.strParamCnt, str_argc); // Total string params
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams(): str_argc: " + str_argc + "");

	var cert_argc = 0;
	var signerX509CertList = multiSigJsonParams.signerX509CertList || [];
	for (var i = 0; i < signerX509CertList.length; i++) {
	    var signerX509Cert = signerX509CertList[i];
	    if (!signerX509Cert) {
		continue;
	    }
	    if (pkiParams) {
		pkiParams.setISupportAtIndex((cert_argc+1), signerX509Cert); cert_argc++;
	    }
	}
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams(): cert_argc: " + cert_argc + "");
	dialogParams.SetInt(XmlMultiDSigUtil.dialogIntParamPositions.certCnt, cert_argc); // total number of certs

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams():................End.");
    },

    multiSigDialogParams2JsonParams : function (aDialogParams)
    {
	// XmlMultiDSigUtil.mMaxLogLevel = 9;
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.multiSigDialogParams2JsonParams():................Start.");

	var dialogParams = aDialogParams;

	var strParamCnt = dialogParams.GetInt(XmlMultiDSigUtil.dialogIntParamPositions.strParamCnt);
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams(): strParamCnt: " + strParamCnt + "");
	if (strParamCnt <= 0) {
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.initParamsOnLoad():................End(2).");
	    var dialogParams = {};
	    return dialogParams;
	}


	var inXMLFilePath = dialogParams.GetString(XmlMultiDSigUtil.dialogStrParamPositions.inputXmlDocFilePath);
	var outXMLFilePath = dialogParams.GetString(XmlMultiDSigUtil.dialogStrParamPositions.outXmlDocFilePath);
	var contentElementURI = dialogParams.GetString(XmlMultiDSigUtil.dialogStrParamPositions.contentElementURI);
	if (!contentElementURI) {
	    contentElementURI = "";
	}

	var signatureType = dialogParams.GetString(XmlMultiDSigUtil.dialogStrParamPositions.signatureType);
	if (!signatureType) {
	    signatureType = "ENVELOPED_SIGNATURE";
	}

	var credentialType = dialogParams.GetString(XmlMultiDSigUtil.dialogStrParamPositions.credentialType);
	if (!credentialType) {
	    credentialType = "certificate";
	}

	var canonicalizationMethod = dialogParams.GetString(XmlMultiDSigUtil.dialogStrParamPositions.canonicalizationMethod);
	if (!canonicalizationMethod) {
	    canonicalizationMethod = "http://www.w3.org/TR/2001/REC-xml-c14n-20010315";
	}

	var signerCnt = 0;
	var signerCntStr = dialogParams.GetString(XmlMultiDSigUtil.dialogStrParamPositions.totSignerX509CertCnt);
	if (signerCntStr) {
	    signerCnt = parseInt(signerCntStr);
	}

	var pkiParams = null;
	var signerX509CertList = [];
	try {
	    pkiParams = dialogParams.QueryInterface(Components.interfaces.nsIPKIParamBlock);
	} catch (ex) {
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.initParamsOnLoad(): pkiParams.getISupportAtIndex() failed - ex: "+ ex + "");
	    signerCnt = 0;
	}

	var certCnt = 0;
	try {
	    certCnt = dialogParams.GetInt(XmlMultiDSigUtil.dialogIntParamPositions.certCnt);
	    if (!certCnt) {
		certCnt = 0;
	    }
	} catch (ex) { }
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.multiSigDialogParams2JsonParams(): certCnt: " + certCnt);
	for (var i = 0; i < certCnt; i++) {
	    var signerX509Cert = null;
	    var x509Cert = pkiParams.getISupportAtIndex(i + 1);
	    XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.multiSigDialogParams2JsonParams(): i: " + i + " x509Cert: " + x509Cert);
	    if (x509Cert) {
		signerX509Cert = x509Cert.QueryInterface(Components.interfaces.nsIX509Cert);
		signerX509CertList.push(signerX509Cert);
	    }
	}

	var multiSigJsonParams = {
		"inputXmlDocFilePath"	: inXMLFilePath,
		"outXmlDocFilePath"	: outXMLFilePath,
		"contentElementURI"	: contentElementURI,
		"signatureType"		: signatureType,
		"credentialType"	: credentialType,
		"canonicalizationMethod" : canonicalizationMethod,
		"totSignerX509CertCnt"	: signerCnt,
		"signerX509CertList"	: signerX509CertList
	};

	// XmlMultiDSigUtil.dumpMultiSigJsonParams(multiSigJsonParams);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.multiSigDialogParams2JsonParams():................End.");
	return multiSigJsonParams;
    },

    getTestMultiSigJsonParams : function ()
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getTestMultiSigJsonParams():................Start.");

	var inXMLFilePath = "/home/subrata/XMLDOC/AMSTestDoc_Unsigned_NoUsers.xml";
	var outXMLFilePath = "/tmp/XMLDSigTool/Signed_AMSTestDoc.xml";
	var contentElementURI = "#Product_1";

	var credentialType = "";
	var canonicalizationMethod = "http://www.w3.org/TR/2001/REC-xml-c14n-20010315";
	var signerCnt = 0;
	var signerX509CertList = [];
	var signerCertNicknameList = ["DocSIgner2", "AEPKTestUser1"];

	for (var i = 0; i < signerCertNicknameList.length; i++) {
	    var signerCertNickname = signerCertNicknameList[i];
	    try {
		signerCert = XmlMultiDSigUtil.mX509CertDB.findCertByNickname(null, signerCertNickname);
		signerX509CertList.push(signerCert);
	    } catch (ex) {
		 XmlMultiDSigUtil.logError("XmlMultiDSigUtil.getTestMultiSigJsonParams(): failed to find signer cert: " + signerCertNickname + " - ex: " + ex + "");
	    }
	}
	if (signerX509CertList.length > 0) {
	    credentialType = "certificate";
	    signerCnt = signerX509CertList.length;
	}

	var multiSigJsonParams = {
		"inputXmlDocFilePath"	: inXMLFilePath,
		"outXmlDocFilePath"	: outXMLFilePath,
		"contentElementURI"	: contentElementURI,
		"credentialType"	: credentialType,
		"canonicalizationMethod" : canonicalizationMethod,
		"totSignerX509CertCnt"	: signerCnt,
		"signerX509CertList"	: signerX509CertList
	};

	// XmlMultiDSigUtil.dumpMultiSigJsonParams(multiSigJsonParams);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getTestMultiSigJsonParams():................End.");
	return multiSigJsonParams;
    },

    getTestMultiSigDialogParams : function ()
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getTestMultiSigDialogParams():................Start.");

	var pkiParams = Components.classes["@mozilla.org/security/pkiparamblock;1"]
				.createInstance(Components.interfaces.nsIPKIParamBlock);
	var dialogParams = pkiParams.QueryInterface(Components.interfaces.nsIDialogParamBlock);

	pkiParams.setISupportAtIndex(1, null);	// 
	dialogParams.SetInt(0, 0);		// dialog close opion - accept(1)/cancel(0)
	dialogParams.SetInt(1, 0);		// number of string paramaters
	dialogParams.SetInt(2, 0);		// number of certs

	var multiSigJsonParams = XmlMultiDSigUtil.getTestMultiSigJsonParams();

	XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams(multiSigJsonParams, dialogParams);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getTestMultiSigDialogParams():................End.");
	return dialogParams;
    },


    getXmlDSigDialogParams : function (aDSigJsonParams)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getXmlDSigDialogParams():................Start.");

	var pkiParams = Components.classes["@mozilla.org/security/pkiparamblock;1"]
				.createInstance(Components.interfaces.nsIPKIParamBlock);
	var dialogParams = pkiParams.QueryInterface(Components.interfaces.nsIDialogParamBlock);

	pkiParams.setISupportAtIndex(1, null);	// 
	dialogParams.SetInt(0, 0);		// dialog close opion - accept(1)/cancel(0)
	dialogParams.SetInt(1, 0);		// number of string paramaters
	dialogParams.SetInt(2, 0);		// number of certs

	// var multiSigJsonParams = XmlMultiDSigUtil.getTestMultiSigJsonParams();

	XmlMultiDSigUtil.multiSigJsonParams2ToDailogParams(aDSigJsonParams, dialogParams);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getXmlDSigDialogParams():................End.");
	return dialogParams;
    },

    getXmlDSigDialogParamsByArgs : function (aInputXmlDocFilePath, aOutXmlDocFilePath, aSignerX509cert)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getXmlDSigDialogParamsByArgs():................Start.");

	var dsigJsonParams = {};
	if (aInputXmlDocFilePath && (aInputXmlDocFilePath != "")) {
	    dsigJsonParams.inputXmlDocFilePath = aInputXmlDocFilePath;
	}
	if (aOutXmlDocFilePath && (aOutXmlDocFilePath != "")) {
	    dsigJsonParams.outXmlDocFilePath = aOutXmlDocFilePath;
	}
	dsigJsonParams.totSignerX509CertCnt = 0;
	dsigJsonParams.signerX509CertList = [];
	if (aSignerX509cert) {
	    dsigJsonParams.credentialType = "certificate";
	    dsigJsonParams.totSignerX509CertCnt = 1;
	    dsigJsonParams.signerX509CertList.push(aSignerX509cert);
	}
	// XmlMultiDSigUtil.dumpMultiSigJsonParams(dsigJsonParams);

	var dialogParams = XmlMultiDSigUtil.getXmlDSigDialogParams(dsigJsonParams);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getXmlDSigDialogParamsByArgs():................End.");
	return dialogParams;
    },

    getDefaultmultiSigJsonParams : function() 
    {
	var multiSigJsonParams = {
		"inputXmlDocFilePath"		: "",
		"outXmlDocFilePath"		: "",
    		"contentElementURI"		: "",
    		"signatureType"			: "ENVELOPED_SIGNATURE",
    		"credentialType"		: "certificate",
    		"canonicalizationMethod"	: "http://www.w3.org/TR/2001/REC-xml-c14n-20010315",
		"totSignerX509CertCnt"		: 0,
		"signerX509CertList"		: []
		};
    	return multiSigJsonParams;
    },

    getXmlSigJSONParams : function (aRootXULElem)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getXmlSigJSONParams():................Start.");

	var /* NodeList */ xmlSigParamList = aRootXULElem.getElementsByAttribute("sigParamName", "*");
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.getXmlSigJSONParams(): xmlSigParamList.length: " + xmlSigParamList.length + "");

	var propKey = "";
	var propValue = "";
	var trueValue = "true";

	var multiSigJsonParams = XmlMultiDSigUtil.getDefaultmultiSigJsonParams();
	for (var i = 0; i < xmlSigParamList.length; i++) {
	    var xmlSigParamElem = xmlSigParamList.item(i);
	    if (xmlSigParamElem.hidden) {
		continue;
	    }
	    propKey = xmlSigParamElem.getAttribute("sigParamName");
	    if ((propKey == null) || (propKey == "")) {
		continue;
	    }
	    propValue = "";
	    if (xmlSigParamElem.tagName == "checkbox") {
		if (xmlSigParamElem.checked) {
		    propValue = "true";
		}
		else {
		    propValue = "false";
		}
	    }
	    else if (xmlSigParamElem.tagName == "radiogroup") {
		propValue = xmlSigParamElem.selectedItem.value;
	    }
	    else if (xmlSigParamElem.tagName == "radio") {
		if (xmlSigParamElem.selected) {
		    propValue = xmlSigParamElem.value;
		}
	    }
	    else if (xmlSigParamElem.tagName == "textbox") {
		propValue = xmlSigParamElem.value;
	    }
	    else if (xmlSigParamElem.value) {
		propValue = xmlSigParamElem.value;
	    }
	    // Initialize the json data with propKey as filedName
	    if (propValue && (propValue != "")) {
		multiSigJsonParams[propKey] = propValue;
	    }
	}
	if (!multiSigJsonParams.totSignerX509CertCnt &&
		multiSigJsonParams.signerX509CertList &&
		multiSigJsonParams.signerX509CertList.length &&
		(multiSigJsonParams.signerX509CertList.length > 0)) {
	    multiSigJsonParams.totSignerX509CertCnt = multiSigJsonParams.signerX509CertList.length;
	}

	// XmlMultiDSigUtil.dumpMultiSigJsonParams(multiSigJsonParams);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.getXmlSigJSONParams():................End.");
	return multiSigJsonParams;
    },

    /* nsIPersistentProperties */
    getXmlSigParamProperties : function (aRootXULElemId)
    {
	var xmlSigProps = Components.classes["@mozilla.org/persistent-properties;1"]
				.createInstance(Components.interfaces.nsIPersistentProperties);

	var propKey;
	var propValue;
	var propOldValue;
	var trueValue = "true";

	// XmlMultiDSigUtil.logDebug("getSignatureFormProperties(): tabPanelId: " + tabPanelId + "");

	var gXmlSignFormElem = document.getElementById(aRootXULElemId);
	var /* NodeList */ xmlSignFormPropElemList = gXmlSignFormElem.getElementsByAttribute("sigParamName", "*");
	// xmlSignFormPropElemList = gXmlSignFormElem.getElementsByAttribute("id", "*");
	for (var i = 0; i < xmlSignFormPropElemList.length; i++) {
	var xmlSigPropElem = xmlSignFormPropElemList.item(i);
	// XmlMultiDSigUtil.logDebug("i: " + i + " tagName: " + xmlSigPropElem.tagName + " id: " + xmlSigPropElem.id + "");

	if (xmlSigPropElem.hidden) {
	    continue;
	}
	var propKey = xmlSigPropElem.getAttribute("sigParamName");
	if ((propKey == null) || (propKey == "")) {
	    continue;
	}
	propValue = "";
	if (xmlSigPropElem.tagName == "checkbox") {
	    if (xmlSigPropElem.checked) {
		propValue = trueValue;
	    }
	}
	else if (xmlSigPropElem.tagName == "radiogroup") {
	    propValue = xmlSigPropElem.selectedItem.value;
	}
	else if (xmlSigPropElem.tagName == "radio") {
	    if (xmlSigPropElem.selected) {
		// propValue = xmlSigPropElem.value;
		propValue = xmlSigPropElem.value;
	    }
	}
	else if (xmlSigPropElem.value) {
	    propValue = xmlSigPropElem.value;
	}
	else {
	    var tmpVal = xmlSigPropElem.getAttribute("value");
	    if (tmpVal) {
		propValue = tmpVal;
	    }
	}
	if (propValue != "") {
	    propOldValue = xmlSigProps.setStringProperty(propKey, propValue);
	}
	// XmlMultiDSigUtil.logDebug("id: " + xmlSigPropElem.id + " propKey: " + propKey + " = " + propValue + "");
	}
	// XmlMultiDSigUtil.logDebug(xmlSigProps, "xmlSignatureParams:");
	return xmlSigProps;
    },

    autoSelectOutFile : function (aFilePickerElem, aFileName)
    {
	// const PERMS_FILE      = 0644;
	const PERMS_DIRECTORY = 0755;

	var outFile = null;
	try {
	    var tmpDirFile = aFilePickerElem.fileDisplayDirFile;
	    if (!tmpDirFile) {
		tmpDirFile = aFilePickerElem.getTmpDir("XMLDSigTool");
	    }
	    if (!tmpDirFile.exists()) {
		tmpDirFile.create(Components.interfaces.nsILocalFile.DIRECTORY_TYPE, PERMS_DIRECTORY);
	    }
	    outFile = aFilePickerElem.autoSelectFile(tmpDirFile, aFileName);
	    if (outFile == null) {
		XmlMultiDSigUtil.logError("XmlMultiDSigUtil.autoSelectOutFile(): failed to get outFile.");
		return outFile;
	    }
	} catch (ex) {
	    XmlMultiDSigUtil.logError("XmlMultiDSigUtil.autoSelectOutFile(): failed to create outFile - ex : " + ex + "");
	    throw ex;
	}
	// XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.autoSelectOutFile(): outFile: " + outFile.path + "");
	return outFile;
    },

    loginToCertToken : function (aCert, force)
    {
	var tokendb = Components.classes["@mozilla.org/security/pk11tokendb;1"]
				.getService(Components.interfaces.nsIPK11TokenDB);
	var keyTokenName = "" + aCert.tokenName;
	var /* nsIPK11Token */ token = tokendb.findTokenByName(keyTokenName);
	if (token == null) {
	    return;
	}
	var forceLogin = ((force == null) ? false : force);
	token.login(forceLogin);
    },

    dumpProperties : function (/* nsIPersistentProperties */ aProps, msg)
    {
	var propKey;
	var propValue;
	var propOldValue;

	if (msg) {
	    XmlMultiDSigUtil.dump(msg + " ");
	}
	XmlMultiDSigUtil.dump(" Properties {\n");
	var propEnum = aProps.enumerate();
	while (propEnum.hasMoreElements ()) {
	    var propElem = propEnum.getNext ();
	    var propItem = propElem.QueryInterface (Components.interfaces.nsIPropertyElement);
	    var propKey = propItem.key;
	    var propValue = propItem.value;
	    XmlMultiDSigUtil.dump("    " + propKey + " = " + propValue + "\n");
	}
	XmlMultiDSigUtil.dump("}\n");
    },

    trimString : function (aStr)
    {
	// return newStr;
	var sInString = aStr.replace( /^\s+/g, "" );// strip leading
	return sInString.replace( /\s+$/g, "" );// strip trailing
    },

    cmdAddSignerToXmlDoc : function (aXmlDoc, aMultiSigJsonParams)
    {
	// XmlMultiDSigUtil.mMaxLogLevel = 9;
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.cmdAddSignerToXmlDoc():..................Start.");

	var presignedXmlDoc = XmlMultiDSigUtil.addSignatureElemToXmlDoc(aXmlDoc, aMultiSigJsonParams);

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.cmdAddSignerToXmlDoc():..................End.");
	return presignedXmlDoc;
    },

    cmdSignXmlDoc : function (unsignedXmlDocFilePath, signedXmlDocFilePath, aSignerX509cert)
    {
	// XmlMultiDSigUtil.mMaxLogLevel = 9;
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.cmdSignXmlDoc():..................Start.");

	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.cmdSignXmlDoc():unsignedXmlDocFilePath: " + unsignedXmlDocFilePath + "");
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.cmdSignXmlDoc():signedXmlDocFilePath: " + signedXmlDocFilePath + "");

	// We are now ready to sign the unsigned XML doc file.

	dialogParams = XmlMultiDSigUtil.getXmlDSigDialogParamsByArgs(
					unsignedXmlDocFilePath,
					signedXmlDocFilePath,
					aSignerX509cert
					);
	/*
	var dialogParams = Components.classes["@mozilla.org/embedcomp/dialogparam;1"]
				.createInstance(Components.interfaces.nsIDialogParamBlock);
	var argc = 0;
	dialogParams.SetString(argc, unsignedXmlDocFilePath); argc++;
	dialogParams.SetString(argc, signedXmlDocFilePath); argc++;
	dialogParams.SetInt(0, argc);
	*/
	window.openDialog('chrome://xmldsigffext/content/xmldsig/XmlDigitalSigTool.xul',
				'_blank', 'chrome,centerscreen,resizable,modal,titlebar',
				dialogParams
				);
	var retVal = dialogParams.GetInt(0);
	if (retVal == 0) {
	    alert("Failed to sign the input XML doc");
	    return false;
	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.cmdSignXmlDoc():..................End.");
	return true;
    },

    cmdVerifySignedXmlDoc : function (signedXmlDocFilePath)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.cmdVerifySignedXmlDoc():..................Start.");
	
	XmlMultiDSigUtil.logDebug("XmlMultiDSigUtil.cmdVerifySignedXmlDoc():signedXmlDocFilePath: " + signedXmlDocFilePath + "");

	// We are now ready to sign the unsigned XML doc file.

	var dialogParams = Components.classes["@mozilla.org/embedcomp/dialogparam;1"].
			createInstance(Components.interfaces.nsIDialogParamBlock);
	var argc = 0;
	dialogParams.SetString(argc, signedXmlDocFilePath); argc++;
	dialogParams.SetInt(0, argc);
	window.openDialog('chrome://xmldsigffext/content/xmldsig/XmlDigitalSigTool.xul',
			'_blank', 'chrome,centerscreen,resizable,modal,titlebar', dialogParams);

	var retVal = dialogParams.GetInt(0);
	if (retVal == 0) {
	    alert("Failed to sign the input XML doc");
	    return false;
	}

	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.cmdVerifySignedXmlDoc():..................End.");
	return true;
    },
 
    verifySignedXMLFileWithX509Cert : function (/* nsIFile */ aSignedXMLFile)
    {
	XmlMultiDSigUtil.logTrace("XmlMultiDSigUtil.verifySignedXMLFileWithX509Cert():................Start.");

	var verficationPassed = false;
	var /* nsIX509Cert */ docSignerCert = null;

	var xmlSigProps = Components.classes["@mozilla.org/persistent-properties;1"]
				.createInstance(Components.interfaces.nsIPersistentProperties);
	try {
	   var signerCertObj = new Object();
	   signerCertObj.value = null;
	   verficationPassed = XmlMultiDSigUtil.mXmlDSigTool.verifySignedXMLFileByX509Cert(
				aSignedXMLFile,
				signerCertObj,
				xmlSignatureProps
				);
	    docSignerCert  = signerCertObj.value;
	} catch (ex) {
	    XmlMultiDSigUtil.logError("verifySignedXMLFileWithX509Cert (): failed to verify signed XML file - ex : " + ex + "");
	    throw ex;
	}

	var verifResult = {
	    signerCert : docSignerCert,
	    verficationPassed : verficationPassed
	};

	XmlMultiDSigUtil.logTrace("xmldsigffext_verifySignedXMLFileWithX509Cert():................End.");
	return verifResult;
    },


    /*
    // Usage of verifySignedXMLFilePath(): 
    var signedXMLFilePath = ""; // TODO: 
    var verifResult = verifySignedXMLFilePath(signedXMLFilePath); 
    if (verifResult.verficationPassed) {
	 alert("XML Digital Signature Varification passed succesfully for file: " + signedXMLFilePath );
	 // TODO: Show the signer certficate 
    }
    else {
	alert("XML Digital Signature Varification failed for file: " + signedXMLFilePath);
    }
    */
    verifySignedXMLFilePath : function (aSignedXMLFilePath)
    {
	var verifResult = {};

	var /* nsIFile */ signedXmlFile = Components.classes["@mozilla.org/file/local;1"]
						.createInstance(Components.interfaces.nsILocalFile);
	signedXmlFile.initWithPath(aSignedXMLFilePath);

	verifResult = verifySignedXMLFileWithX509Cert(signedXmlFile);
	return verifResult;
    },

    lastMethod : function()
    {
    }

};

