/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is  Avaya Labs Research, Avaya Inc. Code.
 *
 * The Initial Developer of the Original Code is
 *   Subrata Mazumdar, Avaya Labs Research, Avaya Inc.
 * Portions created by the Initial Developer are Copyright (C) 2007
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Subrata Mazumdar (mazum@avaya.com)
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

if (typeof avxmldsig == 'undefined') {
    var avxmldsig = {};
}
if (typeof avxmldsig.dsigtool == 'undefined') {
    avxmldsig.dsigtool = {};
}


window.addEventListener(
  "load",
  function (e) {
    avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.init();
    avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.applyHtmlDSigPolicy(e);
  },
  false
);

avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay = {

    mMaxLogLevel		: 2,
    log : function(level, msg)
    {
	if (level > avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.mMaxLogLevel) {
            return;
        }
        dump(msg + "\n");
    },

    logError : function(msg)
    {
        avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.log(2, msg);
    },
    logTrace : function(msg)
    {
        avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.log(4, msg);
    },
    logDebug : function(msg)
    {
        avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.log(8, msg);
    },

    checkXPCOMBinaryComponent : function (interfaceId, classId, isService)
    {
    	if (!Components.interfaces[interfaceId] || !Components.classes[classId]) {
	    var msg = "Failed to load XPCOM implementation (" + classId + ") of " + interfaceId + " interface.";
    	    alert(msg);
    	    dump(msg + "\n");
	    return false;
	}
	if (isService) {
	    var serviceObj = null;
	    try {
	    	serviceObj = Components.classes[classId].getService(Components.interfaces[interfaceId]);
	    } catch (ex) { }
	    if (!serviceObj) {
	    	var msg = "Failed to instantiate XPCOM implementation (" + classId + ") of " + interfaceId + " interface.";
    	    	alert(msg);
    	    	dump(msg + "\n");
		return false;
	    }
	}

	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logDebug("XMLDSigFFExtToolbarOverlay.checkXPCOMBinaryComponent(): successfully loaded XPCOM implementation (" + classId + ") of " + interfaceId + " interface " + (isService ? "as service" :"") + ".");

	return true;
    },

    checkXmlDSigBinaryComponents : function ()
    {

	const XPILoadErrorMessage1 = "Failed to load shared librararies and/or register XPCOM componenets for XML Digital Signature tool ."
	const XPILoadErrorMessage2 = "Make sure that DLL (shared libraries) for Apache Xerces-C++ is in your PATH (LD_LIBRARY_PATH) environment variable.";

	var xpcomABI = Components.classes["@mozilla.org/xre/app-info;1"]
				.getService(Components.interfaces.nsIXULRuntime).XPCOMABI;
	// dump("XMLDSigFFExtToolbarOverlay.checkXmlDSigBinaryComponents(): nsIXULRuntime.XPCOMABI: " + xpcomABI + "\n");

	var errmsg1 = XPILoadErrorMessage1;
	var errmsg2 = XPILoadErrorMessage2;
	var errmsg3 = "";

	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXPCOMBinaryComponent("nsIX509CertDB", "@mozilla.org/security/x509certdb;1", true)) {
    	    dump(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
    	    alert(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
	    return false;
	}
	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXPCOMBinaryComponent("alrIX509CertDBUtil", "@avaya.com/pkm/x509certdbutil;1", true)) {
    	    dump(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
    	    alert(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
	    return false;
	}
	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXPCOMBinaryComponent("alrIXPXmlDSigTool", "@avaya.com/pkm/xmldsigtool;1", true)) {
    	    dump(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
    	    alert(errmsg1 + "\n" + errmsg2 + "\n" + errmsg3 + "\n");
	    return false;
	}

	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logDebug("XMLDSigFFExtToolbarOverlay.checkXmlDSigBinaryComponents(): XMLDSigTool binary components were successfully loaded/registered.");
	return true;
    },

    checkSAMLTokenBinaryComponents : function ()
    {
	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXmlDSigBinaryComponents()) {
	    return false;
	}

	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXPCOMBinaryComponent("alrIXPSamlTokenSvc", "@avaya.com/pkm/samltokensvc;1", true)) {
	    return false;
	}

	return true;
    },

    applyHtmlDSigPolicy : function ()
    {
	// Note:  
	// Enable HTML Document signing conditinally. 
	// HTML Document signing is enabled if the extensions.avxmldsig.htmldsig.enabled preference is
	// set to true.
	// Also, HTML Document signing is disabled if the network.proxy.type is not 0 (direct to internet).
	// TODO: Figure out how to compute or validate signature in background thread 
	// without holding up the browser's main thread.

    	avxmldsig.dsigtool.PrefUtil.init();

	var  htmlDSigEnabled = avxmldsig.dsigtool.PrefUtil.getPreference("extensions.avxmldsig.htmldsig.enable", false);
	var  browserProxyType = avxmldsig.dsigtool.PrefUtil.getPreference("network.proxy.type", 0);
	var  htmlDSigProxyOverride = avxmldsig.dsigtool.PrefUtil.getPreference("extensions.avxmldsig.htmldsig.proxy.override", false);
	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logDebug("xmldsigffextOverlay_applyHtmlDSigPolicy(): " + 
			" htmlDSigEnabled: " + htmlDSigEnabled + " " +
			" browserProxyType: " + browserProxyType + " " +
			" htmlDSigProxyOverride: " + htmlDSigProxyOverride + " "
			);

	var htmlDSigMenuItemIdList = [
		"htmlDSigFFExtFormMenuItem",
		"xmldsigffext.cmd.toolbar.htmldsig"
		/*
		 ,
		"htmlDSigTBExtFormMenuItem",
		"htmlDSigSBExtFormMenuItem"
		*/
		];
	// avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logDebug("xmldsigffextOverlay_applyHtmlDSigPolicy(): browserProxyType: " + browserProxyType + "");

	const  disabledTooltiptext = "HTML Signing is not supported for browser enabled with proxy based HTTP access. You can override this poilcy by setting override option/preference to 'true'.";

	for (var i = 0; i < htmlDSigMenuItemIdList.length; i++) {
	    var htmlDSigMenuItemId = htmlDSigMenuItemIdList[i];
	    var htmlDSigMenuItemElem = document.getElementById(htmlDSigMenuItemId);
	    if (!htmlDSigMenuItemElem) {
		// avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logDebug("xmldsigffextOverlay_applyHtmlDSigPolicy(): failed to find menu-item: " + htmlDSigMenuItemId + "");
		continue;
	    }
	    /*
	    avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logDebug("xmldsigffextOverlay_applyHtmlDSigPolicy(): " + 
		"htmlDSigEnabled: " + htmlDSigEnabled + " " + 
		"browserProxyType: " + browserProxyType + " " + 
		"htmlDSigProxyOverride: " + htmlDSigProxyOverride + " " + 
		"htmlDSigMenuItemId: " + htmlDSigMenuItemId + " " + 
		"htmlDSigMenuItemElem: " + htmlDSigMenuItemElem +
		"");
	    */
    
	    if (!htmlDSigEnabled) {
		htmlDSigMenuItemElem.setAttribute("hidden", true);
		continue;
	    }
	    htmlDSigMenuItemElem.removeAttribute("hidden");

	    if ((browserProxyType > 0) && !htmlDSigProxyOverride) {
		var disabledTooltip = htmlDSigMenuItemElem.getAttribute("disabledTooltip");
		if (!disabledTooltip) {
		    disabledTooltip = disabledTooltiptext;
		}
		htmlDSigMenuItemElem.setAttribute("disabled", true);
		htmlDSigMenuItemElem.setAttribute("tooltiptext", disabledTooltip);
	    }
	    else {
		htmlDSigMenuItemElem.removeAttribute("disabled");
		htmlDSigMenuItemElem.removeAttribute("tooltiptext");
	    }
	}
    },

    showXmlDSigPreferences : function ()
    {
	var xmlDSigToolOptionsURL = 'chrome://xmldsigffext/content/xmldsigffextOptions.xul';
	window.openDialog(
		xmlDSigToolOptionsURL,
		"xmldsig_options",
		"chrome,toolbar,centerscreen,resizable=yes,titlebar"
		);
    },

    showAboutInfo : function ()
    {
	var xmlDSigToolAboutURL = 'chrome://xmldsigffext/content/about.xul';
	window.openDialog(
		xmlDSigToolAboutURL,
		"xmldsig_options",
		'chrome,centerscreen,resizable=yes,dialog=no,modal,titlebar'
		);
    },

    runXmlDSigTool : function ()
    {
	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logTrace("XMLDSigFFExtToolbarOverlay.runXmlDSigTool():.........Start.");

	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXmlDSigBinaryComponents()) {
	    return;
	}
	var xmlDSigToolDialogURL = 'chrome://xmldsigffext/content/xmldsig/XmlDigitalSigTool.xul';
	window.openDialog(xmlDSigToolDialogURL, '_blank', 'chrome,centerscreen,resizable,dialog=no,titlebar');

	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logTrace("XMLDSigFFExtToolbarOverlay.runXmlDSigTool():.........End.");
    },

    runXmlMultiDSigTool : function ()
    {
	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logTrace("XMLDSigFFExtToolbarOverlay.runXmlMultiDSigTool():.........Start.");

	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXmlDSigBinaryComponents()) {
	    return;
	}
	
	var dialogParams = null;
    	// var dialogParams = XmlMultiDSigUtil.getTestMultiSigDialogParams();

	var xmlDSigToolDialogURL = 'chrome://xmldsigffext/content/xmldsig/XmlMultiDigitalSigTool.xul';
	window.openDialog(xmlDSigToolDialogURL,
				'_blank',
				'chrome,centerscreen,resizable,titlebar',
				dialogParams
				);

	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.logTrace("XMLDSigFFExtToolbarOverlay.runXmlMultiDSigTool():.........End.");
    },

    runXmlDSigEncWizard : function ()
    {
	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXmlDSigBinaryComponents()) {
	    return;
	}
	var xmlDSigEncToolDialogURL = 'chrome://xmldsigffext/content/xmldsig/XmlDSigEncWizard.xul';
	window.openDialog(
		xmlDSigEncToolDialogURL,
		'_blank',
		'chrome,centerscreen,resizable,dialog=no,titlebar'
		);
    },

    runSamlSelfTokenWizard : function ()
    {
	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkSAMLTokenBinaryComponents()) {
	    return;
	}
	var samlTokenToolDialogURL = 'chrome://xmldsigffext/content/samltok/SamlSelfToken.xul';
	window.openDialog(
		samlTokenToolDialogURL,
		'_blank',
		'chrome,centerscreen,resizable,dialog=no,titlebar'
		);
    },

    runHtmlDSigWizard : function ()
    {
	if (!avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.checkXmlDSigBinaryComponents()) {
	    return;
	}
	var htmlDSigToolDialogURL = 'chrome://xmldsigffext/content/htmldsig/HtmlDigitalSigTool.xul';
	window.openDialog(
		htmlDSigToolDialogURL,
		'_blank',
		'chrome,centerscreen,resizable,dialog=no,titlebar'
		);
    },

    showXmlDSigUsageDocument : function ()
    {
	var xmldsigUsageDocumentURL = 'chrome://xmldsigffext/content/firstrun/xmldsigffextFirstrun.html';
	
	var gBrowserExists = true;
	if (typeof(gBrowser) == 'undefined') {
	    gBrowserExists = false;
	}


	if (gBrowserExists) {
	    // Source: http://developer.mozilla.org/en/Code_snippets/Tabbed_browser#Opening_a_URL_in_a_new_tab
	    // Add tab, then make active
	    gBrowser.selectedTab = gBrowser.addTab(xmldsigUsageDocumentURL);
	}
	else {
	    // Source: http://developer.mozilla.org/En/Opening_a_Link_in_the_Default_Browser

	    var extps = Components.classes["@mozilla.org/uriloader/external-protocol-service;1"]
			      .getService(Components.interfaces.nsIExternalProtocolService);

	    // check whether an external handler for a scheme exist
	    if (extps.externalProtocolHandlerExists("chrome")) {
		// first construct an nsIURI object using the ioservice
		var ioservice = Components.classes["@mozilla.org/network/io-service;1"]
			          .getService(Components.interfaces.nsIIOService);
		var uriToOpen = ioservice.newURI(xmldsigUsageDocumentURL, null, null);
    
		// now, open it!
		extps.loadURI(uriToOpen, null);
	    }
	    else {
		dump("showXmlDSigUsageDocument(): ExternalProtocolHandler does not exists for chorme.");
		window.open(
		    xmldsigUsageDocumentURL,
		    'XMLDSIGUsageDocument',
		    'resizable,toolbar=1,location=1,status=1,scrollbars=1,width=900,height=700'
		    );
	    }
	}
    },
 
    getWindowMediator : function ()
    {
	const kWindowMediatorContractID = "@mozilla.org/appshell/window-mediator;1";
	const kWindowMediatorIID = Components.interfaces.nsIWindowMediator;
	const kWindowMediator = Components.classes["@mozilla.org/appshell/window-mediator;1"]
					.getService(Components.interfaces.nsIWindowMediator);
	return kWindowMediator;
    },

    mInitialized : false,
    init : function () 
    {
	if (avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.mInitialized) {
	    return;
	}
	avxmldsig.dsigtool.XMLDSigFFExtToolbarOverlay.mInitialized = true;

    	avxmldsig.dsigtool.PrefUtil.init();
    },

    uninit : function () 
    {
    }
}

